"""
Module for plotting of probability density functions
"""

import numpy as np
from matplotlib.colors import LogNorm
from .baseplot import plt, mpl, Plotting


class PlotProfilePdf(Plotting):
    """ Plot probability density distributions

    :param profdata: ProfileData object list to use
    """

    def __init__(self, profdata):
        super().__init__()
        # Lists for the plotted structures
        self.figlist = []
        self.axlist = []
        self.leglist = []
        # Data structures used in the different plots
        self.prdlist = profdata

    def createfigs(self, poutput):
        """ Create the canvas of figures and axes objects

        :param poutput: Switch to produce output as pdf files
        """
        ntypes = 2  # Number of different plot forms
        nspecs = len(self.prdlist[0].prspec)
        numfigs = nspecs * ntypes
        self.figlist = [plt.figure() for _ in range(numfigs)]
        # One plot window for each species and quantity, one axis object in each
        self.axlist = [fig.add_subplot(111) for fig in self.figlist]
        for iprd, prd in enumerate(self.prdlist):
            toplot = self.axlist[0:nspecs]
            self.plot_pdf_atx((0.4, 0.75), toplot, prd)
            self.plot_pdf_cont(self.axlist[nspecs:2*nspecs], prd)
        self.axlist[0].legend()
        for fig in self.figlist:
            fig.tight_layout()
        if poutput:
            self.figlist[0].savefig("pdfat06{}.pdf".format(self.prdlist[0].fext))

    def plot_pdf_atx(self, xpos, axlist, prd, quantity="Qturb"):
        """ Plot the pdf of a quantity at a certain radial position

        :param xpos: Radial position in fraction of the minor radius
        :param axlist: Axes objects to plot into
        :param prd: ProfileData object to plot from
        :param quantity: string which of the profile quantities should be used.
        Can be any of ns, Ts, omn, omt, Gammaturb, Gammanc, Qturb, Qnc, Piturb, Pinc, jbs
        """

        xmin = (np.abs(prd.xs - xpos[0])).argmin()
        xmax = (np.abs(prd.xs - xpos[1])).argmin()
        for iprs, prs in enumerate(prd.prspec):
            ttrace = np.array(prs.profdata[quantity])[:, xmin:xmax].flatten()
            pdf, binedges = np.histogram(ttrace, bins=50, range=(0, 7), density=True)
            linest = "-."
            try:
                if prd.pnt.include_f0_contr:
                    labstring = "w/ NC"
                    linest = "-"
            except AttributeError:
                labstring = "w/o NC"
            axlist[iprs].plot(binedges[:-1] + binedges[1] / 2, pdf, linest, label=labstring)
            axlist[iprs].set_xlabel(self.titles[quantity])

    def plot_pdf_cont(self, axlist, prd, quantity="Qturb"):
        """ Colormap of the the pdf

        :param axlist: Axes objects to plot into
        :param prd: ProfileData object to plot from
        :param quantity: string which of the profile quantities should be used.
        :returns: List of plotted colormaps (for colorbar)
        """
        for iprs, prs in enumerate(prd.prspec):
            arrquant = np.array(prs.profdata[quantity])
            minim = np.amin(arrquant)
            maxim = np.amax(arrquant)
            pdf = []
            for ix in range(len(prd.xs)):
                xpdf, bins = np.histogram(arrquant[:, ix], range=(minim, maxim),
                                          bins=50, density=True)
                pdf.append(xpdf)
            axlist[iprs].pcolormesh(prd.xs, bins[:-1], np.array(pdf).T, cmap=self.cmap_unidirect,
                                    norm=LogNorm(vmin=0.1, vmax=1))
            axlist[iprs].set_ylabel(self.titles[quantity])
            axlist[iprs].set_xlabel(r"$x/a$")



# -*- coding: utf-8 -*-

from tkinter import END

import matplotlib.pyplot as plt
import numpy as np

import ..putils.averages as averages
import ..putils.derivatives as deriv
from .baseplot import Plotting
from .diagnostic import Diagnostic
import os


class DiagShearingRate(Diagnostic):
    def __init__(self, avail_vars=None, specnames=None):
        super().__init__()
        self.name = 'Shearing rate'
        self.tabs = ['fluxtube', 'xglobal']

        self.help_txt = """Analyze zonal flow and shearing rate.
                        \n
                        \nadd MRS: add MRS to radial plots (def. False)
                        \nx0: at whch position one takes traces (def. center)"""

        self.avail_vars = avail_vars
        self.specnames = specnames

        self.opts = {'MRS': {'tag': "add MRS", 'values': [False, True]},
                     'pos': {'tag': "x0", 'values': None},
                     'GAMs': {'tag': "GAM estimate", 'values': [False, True]}}

        self.set_defaults()

        self.options_gui = Diagnostic.OptionsGUI()

    def set_options(self, run, specnames, out_folder):
        
        self.x_local = run.parameters[0].pardict.get('x_local')
        self.y_local = run.parameters[0].pardict.get('y_local')
        if not self.y_local:
            raise NotImplementedError("Not yet implemented for y global simulations")

#        self.run_data = run_data
        self.specnames = specnames
#        self.geom = run_data.geometry
        self.dx = run.spatialgrid[0].x[1] - run.spatialgrid[0].x[0]
        self.x = run.spatialgrid[0].x
        # using the methods from diagspace and fourier is cumbersome, so by hand
        self.nx0 = run.spatialgrid[0].nx0

        self.kx_pos = run.spatialgrid[0].kx_pos

        self.mypos = self.opts['pos']['value']
        if not self.mypos:
            self.my_pos = np.argmin(np.abs(run.spatialgrid[0].x))

        self.shearing_rate = []

        # toggle the reader
        self.get_needed_vars(['field'])

        return self.needed_vars

    def execute(self, data, parameters, geometry, spatialgrid,  step):

        class ZonalStep:
            def __init__(self, phi_zonal_x, Er_x, vExB_x, omegaExB_x, abs_phi_fs):
                self.phi_zonal_x = phi_zonal_x
                self.Er_x = Er_x
                self.vExB_x = vExB_x
                self.omegaExB_x = omegaExB_x
                self.abs_phi_fs = abs_phi_fs

        data_kxky = data.field.phi(step.time, step.field, parameters)

        if self.x_local:
            # flux-tube version
            data_fsavg = np.squeeze(averages.z_av3d(data_kxky[:, 0, :], geometry))
            phi_zonal_x = self.nx0*np.real_if_close(np.fft.ifft(data_fsavg, axis=-1), tol=1e-14)
            # Er
            Er_x = self.nx0*np.real_if_close(
                np.fft.ifft(-1j*spatialgrid.kx_fftorder*data_fsavg, axis=-1),
                tol=1e-14)
            # ExB velocity
            vExB_x = -Er_x/geometry.Cxy
            # shearing rate
            omegaExB_x = -self.nx0*np.real_if_close(
                np.fft.ifft(np.power(spatialgrid.kx_fftorder, 2)*data_fsavg, axis=-1),
                tol=1e-14)/geometry.Cxy

            self.shearing_rate.append(
                ZonalStep(phi_zonal_x, Er_x, vExB_x, omegaExB_x, np.abs(data_fsavg)))

        elif not self.x_local and self.y_local:
            # global version
            # we compute the radial electric field, then we fsavg it, as in
            # Eq. 5.20 of X.Lapillonne thesis. Then divide by C_xy for ExB
            # velocity and further derive for shearing rate. q profile is taken
            # into account. This is correct for circular plasma only"""

            phi_zonal_x = np.squeeze(averages.z_av3d(data_kxky[:, 0, :], geometry))

            # Er
            Er_x = deriv.compute(self.dx, phi_zonal_x, 1)
            # ExB velocity
            vExB_x = -Er_x/geometry.Cxy
            # shearing rate
            omegaExB_x = spatialgrid[0].x/geometry.q*deriv.compute(
                geometry.q*Er_x/spatialgrid[0].x, 2)/self.dx
#            return  # TODO: This will raise an exception!
            self.shearing_rate.append(
                ZonalStep(phi_zonal_x, Er_x, vExB_x, omegaExB_x, np.abs(data_fsavg)))
    
    def dict_to_mgkdb(self):
        Diag_dict = {}
        Diag_dict['my_pos'] = self.my_pos
            
        Diag_dict['Er_x'] = np.array([x.Er_x for x in self.shearing_rate])
        Diag_dict['omegaExB_x'] = np.array([x.omegaExB_x for x in self.shearing_rate])
        Diag_dict['phi_zonal_x'] = np.array([x.phi_zonal_x for x in self.shearing_rate])
        Diag_dict['vExB_x'] = np.array([x.vExB_x for x in self.shearing_rate])
        
        if self.x_local:
            Diag_dict['x_local'] = 1
            Diag_dict['abs_phi_fs'] = np.array([x.abs_phi_fs[0:len(self.kx_pos)] for x in
                                                                 self.shearing_rate]) 
        else:
            Diag_dict['x_local'] = 0
            Diag_dict['abs_phi_fs'] = 'None'
            
        if self.y_local:
            Diag_dict['y_local'] = 1
        else:
            Diag_dict['y_local'] = 0
            
        return Diag_dict

    def plot(self, time_requested, output=None, out_folder=None, terminal=None, suffix = None):

        def plot_a_map(ax, x, y, f, x_lbl, y_lbl, ttl):
#            cm1 = ax.pcolormesh(x, y, f)
            #            cm1 = ax.contourf(x, y, f,
            #                            100, cmap=self.plotbase.cmap_bidirect)
            ax.set_rasterization_zorder(z=-10)
            ax.set_xlabel(x_lbl)
            ax.set_ylabel(y_lbl)
            ax.set_title(ttl)
#            fig.colorbar(cm1)

        self.plotbase = Plotting()

        x_lbl = r'$x/\rho_{ref}$' if self.x_local else r'x/a'

        if len(time_requested) > 1:
            # some maps
            fig = plt.figure()
            ax = fig.add_subplot(2, 2, 1)
            plot_a_map(ax, time_requested, self.x,
                       np.array([x.phi_zonal_x for x in self.shearing_rate]).T,
                       r'$ t c_{ref}/L_{ref}$ ', x_lbl, r'$ \langle\phi\rangle [c_{ref}/L_{ref}]$')

            ax = fig.add_subplot(2, 2, 2)
            plot_a_map(ax, time_requested, self.x,
                       np.array([x.Er_x for x in self.shearing_rate]).T, r'$t c_{ref}/L_{ref}$',
                       x_lbl, r'$E_r [eT_{ref}/ (\rho^*_{ref})^2 L_{ref}]$')

            ax = fig.add_subplot(2, 2, 3)
            plot_a_map(ax, time_requested, self.x,
                       np.array([x.vExB_x for x in self.shearing_rate]).T, r'$t c_{ref}/L_{ref}$',
                       x_lbl, r'$v_{ExB} [c_{ref} \rho^*_{ref}]$')

            ax = fig.add_subplot(2, 2, 4)
            plot_a_map(ax, time_requested, self.x,
                       np.array([x.omegaExB_x for x in self.shearing_rate]).T,
                       r'$t c_{ref}/L_{ref}$', x_lbl, r'$\omega_{ExB} [c_{ref}/L_{ref}]$')

            if out_folder is not None:
                try:
                    fig.savefig( os.path.join(out_folder, 'ShearingRate_Map{}.png'.format(suffix)) , bbox_inches='tight')
                    plt.close(fig)
                except:
                    pass
            else:
                fig.show()


            # time traces
            fig = plt.figure()
            ax = fig.add_subplot(2 + self.x_local, 1, 1)
            ax.plot(time_requested, np.array([x.vExB_x[self.my_pos] for x in self.shearing_rate]).T)
            ax.set_xlabel(r'$t c_{ref}/L_{ref}$')
            ax.set_ylabel(r'$v_{ExB} [c_{ref} \rho^*_{ref}]$')

            ax = fig.add_subplot(2 + self.x_local, 1, 2)
            ax.plot(time_requested,
                    np.array([x.omegaExB_x[self.my_pos] for x in self.shearing_rate]).T)
            ax.set_xlabel(r'$t c_{ref}/L_{ref}$')
            ax.set_ylabel(r'$\omega_{ExB} [c_{ref} \rho^*_{ref}]$')

            if self.x_local:
                ax = fig.add_subplot(2 + self.x_local, 1, 3)
                ax.plot(time_requested, np.array(
                        [np.sqrt(np.mean(np.power(np.abs(x.omegaExB_x), 2))) for x in
                         self.shearing_rate]).T)
                ax.set_xlabel(r'$t c_{ref}/L_{ref}$')
                ax.set_ylabel(r'$\sqrt{|\omega_{ExB}|^2} [c_{ref}/L_{ref}]$')

                shear_avg = averages.mytrapz(np.array(
                        [np.sqrt(np.mean(np.power(np.abs(x.omegaExB_x), 2))) for x in
                         self.shearing_rate]), time_requested)
                str_out = "ExB shearing rate= {:.3f}".format(shear_avg)
                
                if output:
                    output.info_txt.insert(END, str_out + "\n")
                    output.info_txt.see(END)
                if terminal:
                    print(str_out)
            
            if out_folder is not None:
                fig.savefig( os.path.join(out_folder, 'ShearingRate_TimeTraces{}.png'.format(suffix) ), bbox_inches='tight')
                plt.close(fig)
            else:
                fig.show()

        # zonal spectra
        if self.x_local:
            fig = plt.figure()
            ax = fig.add_subplot(3, 1, 1)
            ax.plot(self.kx_pos, averages.mytrapz(np.array(
                    [x.abs_phi_fs[0:len(self.kx_pos)] for x in
                     self.shearing_rate]), time_requested))
            ax.set_xlabel(r'$k_x \rho_{ref}$')
            ax.set_title(r"$|\langle\phi\rangle|$")
            ax.loglog()

            ax = fig.add_subplot(3, 1, 2)
            ax.plot(self.kx_pos, averages.mytrapz(np.array(
                    [x.abs_phi_fs[0:len(self.kx_pos)] for x in
                     self.shearing_rate]), time_requested))
            ax.set_xlabel(r'$k_x \rho_{ref}$')
            ax.set_xscale("log")

            ax = fig.add_subplot(3, 1, 3)
            ax.plot(self.kx_pos, averages.mytrapz(np.array(
                    [x.abs_phi_fs[0:len(self.kx_pos)] for x in
                     self.shearing_rate]), time_requested))
            ax.set_xlabel(r'$k_x \rho_{ref}$')
            
            if out_folder is not None:
                fig.savefig( os.path.join(out_folder, 'ShearingRate_ZonalSpectra{}.png'.format(suffix) ), bbox_inches='tight')
                plt.close(fig)
            else:
                fig.show()

        # radial plots
        fig = plt.figure()
        ax = fig.add_subplot(3, 1, 1)
        ax.plot(self.x,
                averages.mytrapz(np.array([x.phi_zonal_x for x in self.shearing_rate]),
                                 time_requested))
        ax.set_xlabel(x_lbl)
        ax.set_ylabel(r'$v_{ExB} [c_{ref} \rho^*_{ref}]$')

        ax = fig.add_subplot(3, 1, 2)
        ax.plot(self.x,
                averages.mytrapz(np.array([x.vExB_x for x in self.shearing_rate]), time_requested))
        ax.set_xlabel(x_lbl)
        ax.set_ylabel(r'$v_{ExB} [c_{ref} \rho^*_{ref}]$')

        ax = fig.add_subplot(3, 1, 3)
        ax.plot(self.x,
                averages.mytrapz(np.array([x.omegaExB_x for x in self.shearing_rate]),
                                 time_requested))
        ax.set_xlabel(x_lbl)
        ax.set_ylabel(r'$\omega_{ExB} [c_{ref} \rho^*_{ref}]$')

        fig.tight_layout()
        
        if out_folder is not None:
            fig.savefig(os.path.join(out_folder, 'ShearingRate_Radial{}.png'.format(suffix)), bbox_inches='tight')
            plt.close(fig)
        else:
            fig.show()

    def save(time_requested, output=None, out_folder=None):
        pass

    def finalize():
        pass



""" Classes to do plots of the potential and its derivatives """
import numpy as np
import ..utils.averages as av
from .baseplot import plt, Plotting
import ..utils.fourier as fourier
from ..utils import errors as err
from ..utils.geom import Geometry
import ..data.slices
from ..utils.comm import DiagSpace


class ZonalAverage(pydiag.data.slices.MomFieldSlice):
    """ Create a flux-surface averaged ky=0 time series from the field file

    :param common: CommonData object of the run
    :param xrange: Radial range or kx modes to consider
    :param potential: "phi" or "apar"
    """

    def __init__(self, common, rundatafiles, xrange=(None, None), potential="phi", withext=True):
        self.diagspace = DiagSpace(common.spatialgrid, x_fourier=common.x_local, y_fourier=True,
                                   xrange=xrange, yrange=(0, 1), zavg=True)
        super().__init__(common, rundatafiles=rundatafiles, quantity=potential, species=None,
                         diagspace=self.diagspace,
                         modifier_func=lambda dum: dum)  # Use the unmodified data
        self.cm = common
        self.xrange = xrange
        self.phi0_ext = 0  # For the sinusoidal external potential
        self.nk_ext = 1
        self.k_ext = 1
        self.phase_phi_ext = 0
        self.Cxy = 1  # Geometry correction
        # External potential only plausible with electrostatic potential
        self.withext = withext and (potential == "phi")

    def generate_timeseries(self):
        """ Wrapper for fieldlib: Read the ky=0 component of the potential"""
        if not self.cm.y_local:
            raise NotImplementedError('y global is not supported!')
        super().generate_timeseries()
        try:
            phi0_ext = self.cm.pnt.phi0_ext
            nk_ext = self.cm.pnt.kxind_phi_ext
            phase_phi_ext = self.cm.pnt.phase_phi_ext
            k_ext = nk_ext/self.cm.pnt.lx*2*np.pi
        except AttributeError:
            phi0_ext = 0
            nk_ext = 1
            phase_phi_ext = 0
            k_ext = 1
        geom = Geometry(self.cm)
        try:
            self.Cxy = geom.Cxy[self.diagspace.xslice]
        except TypeError:
            self.Cxy = geom.Cxy
        except AttributeError:
            self.Cxy = 1
        if self.withext:
            if self.diagspace.x_fourier:
                xr_min, xr_max = self.xrange[0], self.xrange[1]
                if not self.xrange[0]:
                    xr_min = 0
                if not self.xrange[1]:
                    xr_max = np.inf  # Just a convenient large number
                if xr_min <= nk_ext <= xr_max:
                    nk_ext_subindex = nk_ext - xr_min
                    self.dataarray[:, -nk_ext_subindex] += -0.5j*phi0_ext*np.exp(1j*phase_phi_ext)
                    self.dataarray[:, +nk_ext_subindex] += 0.5j*phi0_ext*np.exp(-1j*phase_phi_ext)
            else:
                xgrid = self.cm.spatialgrid.x[self.diagspace.xslice]
                self.dataarray += phi0_ext*np.sin(k_ext*xgrid + phase_phi_ext)[:, np.newaxis]
        self.dataarray = np.squeeze(self.dataarray)

    def derivative(self, order=1):
        base = self.dataarray[:, self.diagspace.xslice]
        if self.diagspace.x_fourier:
            deriv = (1j*self.cm.spatialgrid.kx_grid_slice(self.diagspace.xslice)) ** order*base
        else:
            xs = self.cm.spatialgrid.x_a[self.diagspace.xslice]
            deriv = np.gradient(base, xs/self.cm.pnt.rhostar, axis=1)
            for o in range(order - 1):
                deriv = np.gradient(deriv, xs/self.cm.pnt.rhostar, axis=1)
        return deriv


class PlotZonalxt(Plotting):
    """ Class to plot the fields from GENE and their derivatives
    So far: electrostatic potential, xt contour

    Only considers the z-averaged ky=0 mode, so far!
    :param commonlist: CommonData objects of the runs to plot
    :param fieldavlist: ZonalAverage objects of the runs to plot
    """

    def __init__(self, commonlist, fieldavlist):
        super().__init__()
        self.fieldavlist = fieldavlist
        self.cm = commonlist
        self.xs = 0

    def plot_phi_xt(self, withext=True, poutput=False):
        """ Plot a color map of phi in x-t space

        :param withext: Include the sinusoidal external potential
        :param poutput: Write plots into pdf files as well
        """
        for cm, fieldav in zip(self.cm, self.fieldavlist):
            fieldav.generate_timeseries()
            if not np.any(fieldav.dataarray):
                print("No field data present. Aborting plot")
                continue
            # Consider x-local and x-global here
            if cm.x_local:
                phi_ky0_kx = fieldav.dataarray
                wexb_t_kx = fieldav.derivative(order=2)/fieldav.Cxy
                self.xs = cm.spatialgrid.x
                phi_ky0 = fourier.kx_to_x(phi_ky0_kx, cm.pnt.nx0, axis=-1)
                wexb_t = fourier.kx_to_x(wexb_t_kx, cm.pnt.nx0, axis=-1)
            else:
                phi_ky0 = np.real_if_close(fieldav.dataarray)[:, fieldav.diagspace.xslice]
                self.xs = cm.spatialgrid.x_a[fieldav.diagspace.xslice]
                wexb_t = -np.real_if_close(-fieldav.derivative(order=2)/fieldav.Cxy)

            fig1 = plt.figure()
            ax1 = fig1.add_subplot(111)
            absval = np.max(np.abs(phi_ky0))
            ax1.set_xlim((np.min(self.xs), np.max(self.xs)))
            ax1.set_ylim((np.min(fieldav.timearray), np.max(fieldav.timearray)))
            cm1 = ax1.pcolormesh(self.xs, fieldav.timearray, phi_ky0, shading="gouraud",
                                 cmap=self.cmap_bidirect, vmax=absval, vmin=-absval,
                                 rasterized=True)
            if cm.x_local:
                ax1.set_xlabel(r"$x/\rho_s$")
            else:
                ax1.set_xlabel(r"$x/a$")
            ax1.set_ylabel(r"$t/(a/c_s)$")
            fig1.colorbar(cm1)
            ax1.set_title(r"$\langle\phi\rangle$", y=1.03)
            fig1.tight_layout()

            fig2 = plt.figure()
            ax2 = fig2.add_subplot(111)
            absval = np.max(np.abs(wexb_t))
            ax2.set_xlim([np.min(self.xs), np.max(self.xs)])
            ax2.set_ylim([np.min(fieldav.timearray), np.max(fieldav.timearray)])
            cm2 = ax2.pcolormesh(self.xs, fieldav.timearray, wexb_t, shading="gouraud",
                                 cmap=self.cmap_bidirect, vmax=absval, vmin=-absval,
                                 rasterized=True)
            ax2.set_xlabel(r"$x/\rho_s$")
            ax2.set_ylabel(r"$t/(a/c_s)$")
            fig2.colorbar(cm2)
            ax2.set_title(r"$\langle\omega_{E\times B}\rangle$", y=1.03)
            fig2.tight_layout()
            if poutput == 1:
                fig1.savefig('phi_xt{}.pdf'.format(fieldav.cm.fileextension))
                fig2.savefig('wexb_xt{}.pdf'.format(fieldav.cm.fileextension))


class PlotZonalAverage(Plotting):
    """ Diagnostic to investigate zonal flows (the ky=0 mode of the potential)

    :param commonlist: CommonData objects of the runs to plot
    :param fieldavlist: ZonalAverage objects of the runs to plot
    """

    def __init__(self, commonlist, fieldavlist):
        super().__init__()
        self.fieldavlist = fieldavlist
        self.xs = 0
        self.kx = 0
        self.cm = commonlist

    def createfigs(self, withext=False, poutput=False):
        """ Plot the time averaged root mean square of the ExB shear

        This routine separates between the local and global shear calculation and
        handles the general frameworks for the plots
        (all lines in one plot or multiple windows)
        :param withext: Include the external sinusoidal potential
        :param poutput: Generate pdf files of the plots
        """
        figlist = [plt.figure() for i in range(3)]
        axlist = [fig.add_subplot(111) for fig in figlist]
        for cm, fieldav in zip(self.cm, self.fieldavlist):
            fieldav.generate_timeseries()
            if not np.any(fieldav.dataarray):
                print("No field data present. Aborting calculation and plot")
                continue
            # Consider x-local and x-global here
            if cm.x_local:
                self.kx = cm.spatialgrid.kx_grid_slice(fieldav.diagspace.xslice)
                self.rms_omega_local(fieldav, axlist)
            else:
                self.xs = cm.spatialgrid.x[fieldav.diagspace.xslice]
                self.xs = self.xs*cm.pnt.rhostar + cm.pnt.x0
                self.rms_omega_global(fieldav, axlist)
        for fig in figlist:
            fig.tight_layout()
        if poutput:
            figlist[0].savefig('phi_zonal{}.pdf'.format(self.fieldavlist[-1].cm.fileextension))
            figlist[1].savefig('erad_zonal{}.pdf'.format(self.fieldavlist[-1].cm.fileextension))
            figlist[2].savefig('wexb_zonal{}.pdf'.format(self.fieldavlist[-1].cm.fileextension))

    def rms_omega_local(self, fieldav, axlist):
        """ Local version for the ExB shear calculation

        Generates time averaged kx spectra for <phi>, E_r and omega_ExB
        and prints the RMS for omega (order: first x RMS then time average)
        The plots are logarithmic and ignore the kx=0 mode
        (would be NC, but we cannot calculate phi for it anyway),
        as well as the negative kx - which are Hermitian to kx>0 for ky=0 anyway

        :param fieldav: The ZonalAverage object to be plotted
        :param axlist: The list of axes objects to plot into (len(axlist) >= 3)
        """
        kx = self.kx
        phi_ky0 = fieldav.dataarray
        erad_t = -fieldav.derivative(order=1)
        omega_t = -fieldav.derivative(order=2)/fieldav.Cxy
        times = fieldav.timearray
        rms_omega_t = np.sqrt(np.sum(np.square(np.abs(omega_t)), axis=1))
        err_rms_omega = err.windowerr(rms_omega_t, times)[0]
        av_rms_omega = av.mytrapz(rms_omega_t, times)
        print("Rms ExB shear (time average): ", av_rms_omega, " +- ", err_rms_omega)
        phi_tav = av.mytrapz(np.fft.ifftshift(np.abs(phi_ky0)), times)
        erad_tav = av.mytrapz(np.fft.ifftshift(np.abs(erad_t)), times)
        omega_tav = av.mytrapz(np.fft.ifftshift(np.abs(omega_t)), times)

        ax1 = axlist[0]
        ax1.set_xscale('log')
        ax1.set_xlim(xmin=self._minlogaxis(np.abs(kx)), xmax=self._maxlogaxis(np.abs(kx)))
        ax1.set_yscale('log')
        ax1.set_xlabel(r'$k_x \rho_s$')
        ax1.set_ylabel(r"$|\langle \phi \rangle|$")
        ax1.plot(np.fft.ifftshift(kx), phi_tav)
        ax1.set_ylim(ymin=self._minlogaxis(phi_tav), ymax=self._maxlogaxis(phi_tav))

        ax2 = axlist[1]
        ax2.set_xscale('log')
        ax2.set_xlim(xmin=self._minlogaxis(np.abs(kx)), xmax=self._maxlogaxis(np.abs(kx)))
        ax2.set_yscale('log')
        ax2.set_xlabel(r'$k_x \rho_s$')
        ax2.set_ylabel(r"$|\langle \mathbf{E_x} \rangle|$")
        ax2.plot(np.fft.ifftshift(kx), erad_tav)
        ax2.set_ylim(ymin=self._minlogaxis(erad_tav), ymax=self._maxlogaxis(erad_tav))

        ax3 = axlist[2]
        ax3.set_xscale('log')
        ax3.set_xlim(xmin=self._minlogaxis(np.abs(kx)), xmax=self._maxlogaxis(np.abs(kx)))
        ax3.set_yscale('log')
        ax3.set_xlabel(r'$k_x \rho_s$')
        ax3.set_ylabel(r"$|\langle \omega_{\mathbf{E}\times\mathbf{B}} \rangle|$")
        ax3.plot(np.fft.ifftshift(kx), omega_tav)
        ax3.set_ylim(ymin=self._minlogaxis(omega_tav), ymax=self._maxlogaxis(omega_tav))

    def rms_omega_global(self, fieldav, axlist):
        """ Global version for the ExB shear calculation

        Generates time averaged x profiles for <phi>, E_r and omega_ExB
        and prints the RMS and median of absolutes
        for omega (order: first x RMS then time average)

        :param fieldav: The ZonalAverage object to be plotted
        :param axlist: The list of axes objects to plot into (len(axlist) >= 3)
        """
        xs = self.xs
        phi_ky0_x = np.real_if_close(fieldav.dataarray)
        erad_t = -np.real_if_close(fieldav.derivative(order=1))
        wexb_t = -np.real_if_close(fieldav.derivative(order=2)/fieldav.Cxy)
        fullphi_tav = av.mytrapz(phi_ky0_x, fieldav.timearray)
        erad_tav = av.mytrapz(erad_t, fieldav.timearray)
        wexb_tav = av.mytrapz(wexb_t, fieldav.timearray)

        rms_wexb_t = np.sqrt(np.mean(wexb_t ** 2, axis=1))
        err_rms_wexb = err.windowerr(rms_wexb_t, np.array(fieldav.timearray))[0]
        av_rms_wexb = av.mytrapz(rms_wexb_t, fieldav.timearray)
        print("Rms ExB shear (time average): ", av_rms_wexb, " +- ", err_rms_wexb)
        medabs_wexb_t = np.median(np.abs(wexb_t), axis=1)
        av_medabs_wexb = av.mytrapz(medabs_wexb_t, fieldav.timearray)
        print("Median absolute ExB shear (time average): ", av_medabs_wexb)

        ax1 = axlist[0]
        ax1.set_xlabel(r'$x/a$')
        ax1.set_ylabel(r"$\langle \phi \rangle$")
        ax1.plot(xs, fullphi_tav)

        ax2 = axlist[1]
        ax2.set_xlabel(r'$x/a$')
        ax2.set_ylabel(r"$\langle \mathbf{E_x} \rangle$")
        ax2.plot(xs, erad_tav)

        ax3 = axlist[2]
        ax3.set_xlabel(r'$x/a$')
        ax3.set_ylabel(r"$\langle \omega_{\mathbf{E}\times\mathbf{B}} \rangle$")
        ax3.plot(xs, wexb_tav)
        ax3.axhline(color="Black", ls="--", label='_nolegend_')



""" geomfft.py: class preparing input for GENE's generalized Miller equilibrium from GENE outputs"""
import re
import os
import datetime
import numpy as np
import warnings
from collections import OrderedDict
import .geom
import .comm
import ..data.omega_eigenvalue as eigenvalue
import json
import .averages as averages
import matplotlib.pyplot as plt #remove later
import ..diagplots.plot_ball as ballooning
import ..data.nrgdata as nrg
from scipy import interpolate

class GEOMFFT(object):
    """ GEOMFFT class:

    Determines shape parameters for generalized Miller equilibrium

    """

    def __init__(self,common,rundatafiles):
        print ("Called flux surface parametrization tool ... ")
        self.cm = common
        self.geom = pydiag.utils.geom.Geometry(self.cm)
        self.get_shape()

    def get_flux_surface_distance(self,R0,Z0,R,Z):
        nz0 = len(R)
        #poloidal theta (COCOS=2/12); theta is running counter-clockwise
        #in a poloidal plain to the right of the magnetic axis
        theta = np.arctan((Z-Z0)/(R-R0))
        for iz in range(int(nz0/2),nz0-1):
            thetadiff = (theta[iz+1]-theta[iz])
            if thetadiff>0.5*np.pi:
                theta[iz+1]-=np.pi
            elif thetadiff<-0.5*np.pi:
                theta[iz+1]+=np.pi
            thetadiff = (theta[nz0-1-iz-1]-theta[nz0-1-iz])
            if thetadiff>0.5*np.pi:
                theta[nz0-1-iz-1]-=np.pi
            elif thetadiff<-0.5*np.pi:
                theta[nz0-1-iz-1]+=np.pi

        #sorted theta (starting at th=0)
        theta=np.where(abs(theta)<1E-12,0.0,theta)
#        print ('theta = ', theta)
        theta_sort = (theta+2.0*np.pi)%(2.0*np.pi)
        sort_ind = np.argsort(theta_sort)
#        print('sort_ind = ', sort_ind)
        theta_sort = theta_sort[sort_ind]
#        print('theta_sort = ', theta_sort)
        aN = np.sqrt((R[sort_ind]-R0)**2+(Z[sort_ind]-Z0)**2) #/R0

        #now extend aN and theta_sort to definitely cover >= [0.0,2*pi]
        theta_sort = np.append(theta_sort[-1]-2.0*np.pi,theta_sort)
        theta_sort = np.append(theta_sort,theta_sort[1]+2.0*np.pi)
        aN = np.append(aN[-1],aN)
        aN = np.append(aN,aN[1])

        return {"theta" : theta,          #poloidal angle for original data
                "sort_ind" : sort_ind,    #sort index for original data
                "aN" : aN,                #sorted+ext. flux surface radius funct.
                "theta_sort" : theta_sort #sorted+ext. theta
                }


    def get_shape(self):
        """ Determines shape parameters for generalized Miller equilibrium """

        print ("Determining flux surface shape ...")
        nz0 = self.cm.pnt.nz0

        #determine max/min indices (used several times)
        self.maxR_ind = np.argmax(self.geom.R)
        self.minR_ind = np.argmin(self.geom.R)

        #determine flux surface center
        self.R0 = 0.5*(max(self.geom.R)+min(self.geom.R))
        self.Z0 = 0.5*(max(self.geom.Z)+min(self.geom.Z))
        print ("(R0,Z0) = (", self.R0, ",", self.Z0,")")

        #minor radius of flux surface
        self.r = 0.5*(max(self.geom.R)-min(self.geom.R))

        #New reference length
        self.Lref = self.R0

        #conversion factor
        self.dxdr = 2.0/(self.geom.dxdR[self.maxR_ind]/self.geom.gxx[self.maxR_ind]-\
                         self.geom.dxdR[self.minR_ind]/self.geom.gxx[self.minR_ind])

        #center result (r0)
        res_c=self.get_flux_surface_distance(self.R0,self.Z0,self.geom.R,self.geom.Z)
        self.theta=res_c["theta"]
        aN = res_c["aN"]/self.Lref
        theta_sort=res_c["theta_sort"]
        sort_ind=res_c["sort_ind"]

        #delta_r for finite differences (should be small)
        delta_r = 0.00001 #*self.r/self.R0

        dRdr = self.geom.dxdR/self.geom.gxx*self.dxdr
        dZdr = self.geom.dxdZ/self.geom.gxx*self.dxdr

        #left: r0-dr
        res_l=self.get_flux_surface_distance(self.R0,self.Z0,
                (self.geom.R-delta_r*dRdr),(self.geom.Z-delta_r*dZdr))
        aNl = res_l["aN"]/self.Lref
        thetal = res_l["theta_sort"]

        #right: r0+dr
        res_r=self.get_flux_surface_distance(self.R0,self.Z0,
                (self.geom.R+delta_r*dRdr),(self.geom.Z+delta_r*dZdr))
        aNr = res_r["aN"]/self.Lref
        thetar = res_r["theta_sort"]


        #equidistant theta (COCOS=2/12) with somewhat higher resolution for fourier transforms etc
        nth = 10*nz0
        theta_grid = (np.arange(0.0,nth,1.0))*2.0*np.pi/nth

        #interpolate to this equidist. grid with proper order (interpol requires increasing arr)
        fc = interpolate.interp1d(theta_sort,aN,kind='cubic') #,fill_value='extrapolate')
        aN_th = fc(theta_grid)
        fl = interpolate.interp1d(thetal,aNl,kind='cubic') #,fill_value='extrapolate')
        aNl_th = fl(theta_grid)
        fr = interpolate.interp1d(thetar,aNr,kind='cubic') #,fill_value='extrapolate')
        aNr_th = fr(theta_grid)

        #using 2nd order centered scheme for derivative
        aN_dr_th = (aNr_th-aNl_th)/(2.0*delta_r)*self.Lref

        #Note: Using the following analytic expression for the derivative doesn't work
        #as it neglects variations of theta with increased R,Z
#        aN_dr = ((self.geom.R[sort_ind]-self.R0)*dRdr[sort_ind]+
#                 (self.geom.Z[sort_ind]-self.Z0)*dZdr[sort_ind])/\
#                 (aN*self.R0)


        #now fourier transform
        fft_aN = np.fft.fft(aN_th)
        fft_aN_dr = np.fft.fft(aN_dr_th)

        #keep up to nth_kept modes (typically around 8)
        nth_kept = min(nz0,30)
        self.cN = 2.0*np.real(fft_aN)[0:nth_kept]/nth
        self.cN[0] *= 0.5
        self.sN = -2.0*np.imag(fft_aN)[0:nth_kept]/nth
        self.sN[0] *= 0.5
        self.cN_dr = 2.0*np.real(fft_aN_dr)[0:nth_kept]/nth
        self.cN_dr[0] *= 0.5
        self.sN_dr = -2.0*np.imag(fft_aN_dr)[0:nth_kept]/nth
        self.sN_dr[0] *= 0.5

        #check parametrization
        nind = np.arange(0.0,nth_kept,1.0)
        a_check = []
        a_dr_check = []
        for iz in range(0,nth):
            a_check += [np.sum(self.cN*np.cos(nind*theta_grid[iz])+
                               self.sN*np.sin(nind*theta_grid[iz]))]
            a_dr_check += [np.sum(self.cN_dr*np.cos(nind*theta_grid[iz])+
                                  self.sN_dr*np.sin(nind*theta_grid[iz]))]

        a_dr_check = np.array(a_dr_check)
        err = np.sum(np.abs(a_check-aN_th)/aN_th)/nth
        print ("Relative error of flux surface parametrization: {0:12.6E}".format(err))
        err = np.sum(np.abs(a_dr_check-aN_dr_th)/aN_dr_th)/nth
        print ("Relative error of flux surface derivative parametrization: {0:12.6E}".format(err))


        #Write information to screen
        print("")
        print("&geometry")
        print("magn_geometry = 'miller_general'")
        print("q0   =    ", self.cm.pnt.q0)
        print("shat =    ", self.cm.pnt.shat*self.dxdr*self.r/(self.cm.pnt.x0*self.cm.pnt.Lref))
        print("trpeps =  ", self.r/self.R0)
        print("major_R = ", self.R0/self.Lref)
        print("major_Z = ", self.Z0/self.Lref)
        self.B0 = self.cm.pnt.Bref*self.cm.pnt.Lref*self.cm.pnt.major_R/self.R0
        print("amhd =    ", self.cm.pnt.q0**2*self.cm.pnt.dpdx_pm*\
              (self.cm.pnt.Bref/self.B0)**2*self.dxdr*self.Lref/self.cm.pnt.Lref)
#        print("dpdx_pm = ", self.cm.pnt.dpdx_pm*self.Lref/self.cm.pnt.Lref*(self.cm.pnt.Bref/self.B0)**2*self.dxdr)
        print("")

        print ("cN_m = ",end="")
        for mode in self.cN:
            print ("{0:15.8e},".format(mode),end="")
        print("")
        print ("sN_m = ",end="")
        for mode in self.sN:
            print ("{0:15.8e},".format(mode),end="")
        print("")
        print ("cNdr_m = ",end="")
        for mode in self.cN_dr:
            print ("{0:15.8e},".format(mode),end="")
        print("")
        print ("sNdr_m = ",end="")
        for mode in self.sN_dr:
            print ("{0:15.8e},".format(mode),end="")
        print("")

        print("")
        print("sign_Bt_CW = ", self.cm.pnt.sign_Bt_CW)
        print("sign_Ip_CW = ", self.cm.pnt.sign_Ip_CW)
        print("")

        print("rhostar = -1")
        print("/")
        print("")

        print("")
        print("Lref = ",self.Lref)
        print("Bref = ",self.B0)
        print ("!grad. conversion: ", self.dxdr*self.Lref/self.cm.pnt.Lref)
        print("")

        #Gradient conversions
        print ("Lref_GENE/R0 = ", self.cm.pnt.Lref/self.R0)
        print ("Bref_new/Bref_old = ", self.B0/self.cm.pnt.Bref)
        print ("dx/dr = ", self.dxdr)
        #For comparison with Miller script in GENE:
        print ("drho_tor/dr = ", self.dxdr/self.cm.pnt.Lref)


        #some plot diagnostics
        show_aN = True
        if show_aN:
            plt.plot(theta_sort,aN,'x-',markersize=5,label='aN_orig')
#            plt.plot(theta_grid,aN_th,label='aN_interpol')
            plt.plot(theta_grid,a_check,label='aN_final')
#            plt.plot(theta_sort,aN_dr,'o-',markersize=5,label='aN_dr_orig')
            plt.plot(theta_grid,aN_dr_th,'o-',markersize=5,label='aN_dr interpol')
            plt.plot(theta_grid,a_dr_check,label='aN_dr_final')
            plt.legend()
            plt.show()

        show_flux_surface = True
        if show_flux_surface:
            dr = 0.05*self.r  #radial extension for testing

            #original data
            plt.plot(self.geom.R[sort_ind],self.geom.Z[sort_ind],'o-',markersize=3,label="original")
            plt.plot((self.geom.R[sort_ind]+dr*dRdr[sort_ind]),
                     (self.geom.Z[sort_ind]+dr*dZdr[sort_ind]),
                     'x-',markersize=3,label="original+dr")

            #aN and aN_dr from original data
#            R_check = self.R0+aN*np.cos(theta_sort)*self.R0
#            Z_check = self.Z0+aN*np.sin(theta_sort)*self.R0
#            plt.plot(R_check,Z_check,label="aN")

            if (dr==delta_r):
                R_check = self.R0+aNr*np.cos(thetar)*self.Lref
                Z_check = self.Z0+aNr*np.sin(thetar)*self.Lref
                plt.plot(R_check,Z_check,label="aNr")

            #interpolated aN and aN_dr
#            R_check = self.R0+aN_th*np.cos(theta_grid)*self.R0
#            Z_check = self.Z0+aN_th*np.sin(theta_grid)*self.R0
#           plt.plot(R_check,Z_check,label="aN_th")
#            R_check = self.R0+(aN_th+dr/self.R0*aN_dr_th)*np.cos(theta_grid)*self.R0
#            Z_check = self.Z0+(aN_th+dr/self.R0*aN_dr_th)*np.sin(theta_grid)*self.R0
#            plt.plot(R_check,Z_check,label="aN_dr_th")

            #flux surface from Fourier coefficients
            theta_grid = (np.arange(0.0,nth,1.0))*2.0*np.pi/(nth-1)
            R_check = self.R0+a_check*np.cos(theta_grid)*self.Lref
            Z_check = self.Z0+a_check*np.sin(theta_grid)*self.Lref
            plt.plot(R_check,Z_check,label="Gen. Miller")
            R_check = self.R0+(a_check+dr/self.Lref*a_dr_check)*np.cos(theta_grid)*self.Lref
            Z_check = self.Z0+(a_check+dr/self.Lref*a_dr_check)*np.sin(theta_grid)*self.Lref
            plt.plot(R_check,Z_check,label="Gen. Miller+dr")

            #rest of plotting
            plt.plot(self.R0,self.Z0,'o',markersize=2,color='black')
            plt.xlabel('R/m')
            plt.ylabel('Z/m')
            plt.axis('equal')
            plt.legend()
            plt.show()



import numpy as np
import warnings
#import os
from .ParIO import Parameters


class CommonData(object):
    """ Class containing basic information shared between diagnostics

    :param fileextension: File extension of the run to analyse
    :param starttime: Start of the analysis time window
    :param endtime: End of the analysis time window

    Special choices for starttime and endtime:
    -1: use first time in profile file
    -2: use last time in profile file
    """

    def __init__(self, fileextension, starttime, endtime):
        self.starttime = starttime
        self.endtime = endtime
        self.fileextension = fileextension
        params = Parameters()
        params.Read_Pars( 'parameters{}'.format(fileextension)) 
        self.pars = params.pardict
        self.pnt = params.asnamedtuple()
        # Determine if we are in one of the global versions
        self.x_local = self.pars.get('x_local', True)  # By default, x_local = T is not written
        self.y_local = self.pars.get('y_local', True)  # By default, y_local = T is not written
        self.nonlinear = self.pars.get('nonlinear')
        self.electromagnetic = (self.pars.get('beta', False) != 0)
        self.bpar = self.pars.get("bpar", False)
        self.specnames = [str(self.pars['name{:1d}'.format(nsp + 1)])[1:-1] for nsp in
                          range(self.pnt.n_spec)]
        self.spatialgrid = SpatialGrid(self.pnt, self.x_local, self.y_local)


class SpatialGrid(object):
    """ The numerical grid based on the parameters of a GENE run

    :param pnt: NamedTuple of the run parameters
    :param x_local: Is the run local in radial direction?
    :param y_local: Is the run local in "binormal" direction?
    """

    def __init__(self, pnt, x_local, y_local):
        try:
            kx_center = pnt.kx_center
        except AttributeError:
            kx_center = 0
        self._calc_kxgrid(x_local, y_local, pnt.lx, pnt.nx0, kx_center)
        self._calc_xgrid(x_local, pnt.lx, pnt.nx0, pnt.rhostar, pnt.x0)
        self._calc_kygrid(y_local, pnt.kymin, pnt.nky0, pnt.ky0_ind)
        self._calc_ygrid(y_local, pnt.kymin, pnt.nky0)
        self.z = np.array([-np.pi + 2*np.pi*k/pnt.nz0 for k in range(pnt.nz0)], dtype=np.float64)
        self.kz = 2*np.pi * np.fft.fftfreq(pnt.nz0, d=2*np.pi/pnt.nz0)
        self.nx0 = pnt.nx0

    def _calc_xgrid(self, x_local, lx, nx0, rhostar, x0):
        if x_local:
            xgrid = np.fft.fftfreq(nx0, d=1.0/lx)
            self.x = np.fft.fftshift(xgrid)
            self.x_a = self.x*rhostar + x0
        else:
            self.x = np.array([-lx/2. + lx/(nx0 - 1)*i for i in range(0, nx0)])
            self.x_a = self.x*rhostar + x0

    def _calc_kxgrid(self, x_local, y_local, lx, nx0, kx_center=0):
        self.kxmin = 2*np.pi/lx

        if x_local:
            if y_local:
                kxgridp = [self.kxmin*i for i in range(int((nx0+1)/2)+(1-nx0%2))]
                kxgridn = [self.kxmin*i for i in range(int(-(nx0-1)/2), 0)]
                # positive kx modes
                self.kx_pos = np.array(kxgridp) + kx_center
                self.kx_fftorder = np.array(kxgridp + kxgridn, dtype=np.float64) + kx_center
                # ordered kx array
                self.kx = np.array(kxgridn + kxgridp, dtype=np.float64) + kx_center
            else:
                self.kx = np.array([self.kxmin*i for i in range(nx0)])
        else:
            if y_local:
                self.kx_fftorder = 2*np.pi*np.fft.fftfreq(nx0, d=lx/nx0)
                self.kx = np.fft.ifftshift(self.kx_fftorder)
                self.kx_pos = self.kx_fftorder[:int(nx0/2+1)]
            else:
                warnings.warn("Existing minimal xy-global support may be dropped in the future",
                              DeprecationWarning)
                self.kx_fftorder = 2*np.pi*np.fft.fftfreq(nx0, d=lx/2/nx0)
                self.kx = np.fft.ifftshift(self.kx_fftorder)
                self.kx_pos = self.kx

    def _calc_ygrid(self, y_local, kymin, nky0):
        ly = 2*np.pi/kymin
        if y_local:
            self.y = np.fft.fftshift(np.fft.fftfreq(2*nky0, d=1/ly))
        else:
            self.y = np.array([-ly/2. + ly/nky0*i for i in range(0, nky0)])

    def _calc_kygrid(self, y_local, kymin, nky0, ky0_ind=0):
        if y_local:
            self.ky = np.array([kymin*(j+ky0_ind) for j in range(nky0)], dtype=np.float64)
        else:
            ly = 2*np.pi/kymin
            self.ky = 2*np.pi*np.fft.ifftshift(np.fft.fftfreq(nky0, d=ly/nky0))

    def kx_grid_slice(self, kxslice):
        """Get a full kx grid in FFT order from a slice for positive kx"""
        sliced_kxpos = self.kx_pos[kxslice]
        if self.nx0%2 == 1:
            sliced_kxneg = -sliced_kxpos[-1::-1]
        else:
            if kxslice.stop is None:  # Nyquist mode must be ignored
                sliced_kxneg = -sliced_kxpos[-2::-1]
            else:
                sliced_kxneg = -sliced_kxpos[-1::-1]
        if kxslice.start is None:
            sliced_kxneg = sliced_kxneg[:-1]
        sliced_kx = np.concatenate((sliced_kxpos, sliced_kxneg))
        return sliced_kx


class DiagSpace(object):
    """ Class defining the space that should be used in a diagnostic

    i.e. which intervals in x, y, z, Fourier or configuration space in x,y
    Velocity space is not handled by this class
    The ranges are given as tuples of the starting and stopping index (which is not included)
    Negative indices can be used in the usual Python way (-1 indexing the last element).
    Be aware that while [0:] includes the last element [0:-1] does not!
    Refer to https://docs.scipy.org/doc/numpy/reference/arrays.indexing.html for the details
    how indexing numpy arrays works.
    A binary flag indicates if an average is desired in that direction.
    """

    def __init__(self, rungrid, x_fourier, y_fourier, z_fourier=False, xrange=(None, None), yrange=(None, None),
                 zrange=(None, None), xavg=False, yavg=False, zavg=False):
        """ Standard constructor for DiagSpace

        :param rungrid: SpatialGrid object, supplies the numerical grid of the GENE run
        :param x_fourier: If true, operate in kx space, Fourier transform data if necessary
        :param y_fourier: If true, operate in ky space, Fourier transform data if necessary
        :param z_fourier: If true, operate in kz space, Fourier transform data if necessary
        :param xrange: x or kx index range as a tuple (i_min, i_max). Note that kx refers to the positive kx modes here.
        :param yrange: y or ky index range as a tuple (j_min, j_max)
        :param zrange: z index range as a tuple (k_min, k_max)

        Note that None is also a possible choice for the min and max values due to [:]
        giving the entire array.
        """
        self.xavg = xavg
        self.yavg = yavg
        self.zavg = zavg
        self.x_fourier = x_fourier
        self.y_fourier = y_fourier
        self.z_fourier = z_fourier
        if x_fourier:
            self.xslice = self._create_slice(xrange, rungrid.kx_pos)
        else:
            self.xslice = self._create_slice(xrange, rungrid.x)
        if y_fourier:
            self.yslice = self._create_slice(yrange, rungrid.ky)
        else:
            self.yslice = self._create_slice(yrange, rungrid.y)
        if z_fourier:
            self.zslice = self._create_slice(zrange, rungrid.kz)
        else:
            self.zslice = self._create_slice(zrange, rungrid.z)
        self.xyslice = (self.xslice, self.yslice)
        self.xzslice = (self.xslice, self.zslice)
        self.yzslice = (self.yslice, self.zslice)
        if not (xavg or yavg or zavg):
            self.diagslice = (self.xslice, self.yslice, self.zslice)
        elif xavg and not yavg and not zavg:
            self.diagslice = self.yzslice
        elif xavg and yavg and not zavg:
            self.diagslice = self.zslice
        elif xavg and not yavg and zavg:
            self.diagslice = self.yslice
        elif not xavg and not yavg and zavg:
            self.diagslice = self.xyslice
        elif not xavg and yavg and not zavg:
            self.diagslice = self.xzslice
        elif not xavg and yavg and zavg:
            self.diagslice = self.xslice
        else:
            self.diagslice = None
        self.rungrid = rungrid

    @staticmethod
    def _create_slice(coordrange, grid_direction):
        if grid_direction[coordrange[0]:coordrange[1]].size == 0:
            raise IndexError(
                    "The given range {} is empty or outside of the grid ({}:{}) of the run".format(
                        coordrange, 0, grid_direction.size-1))
        return slice(coordrange[0], coordrange[1])



""" ParIO.py: Contains the class to handle reading and writing of parameter files """
import re
import numpy as np
from collections import namedtuple, OrderedDict


class Parameters(object):
    """ Parameters class:

    Converts a GENE parameters file to a python dictionary and vice versa
    """

    def __init__(self):
        # dictionary for parameters: {parameter: value}
        self.pardict = OrderedDict()
        # dictionary for recording the namelists the parameters belong to: {parameter: namelist}
        self.nmldict = OrderedDict()
        # keep track of all namelists that have been found
        self.namelists = []
        self.spec_nl = (
            'omn', 'omt', 'mass', 'charge', 'dens', 'temp', 'name', 'passive', 'kappa_n', 'kappa_T',
            'LT_center', 'Ln_center', 'LT_width', 'Ln_width', 'prof_type', 'src_prof_type',
            'src_amp', 'src_width', 'src_x0', 'delta_x_n', 'delta_x_T', 'prof_file')

    @staticmethod
    def clearcomments(variable):
        regex = re.compile(r'\s*([-+\'\"\[\];.,/a-zA-Z0-9_\s*]*)\s*!?\s*(.*)')
        result = regex.search(variable)
        if result and result.group(2)[:4] != 'scan':
            return regex.search(variable).group(1)
        else:
            return variable

    def Read_Pars(self, path):
        """ Read parameters file and make it a dict """
        self.pardict.clear()
        self.nmldict.clear()
        # counts species namelists
        countspec = 0
        try:
            with open(path, "r") as parfile:
                # Search file for parameters using regular expressions
                for line in parfile:
                    # Exclude commented lines
                    if re.search(r'\s*!\w*\s*=.*', line) is None:
                        # Check for and count species namelists
                        if re.search(r'^\s*&(.*)', line):
                            # if namelist belongs to a species, append its number to the namelist
                            if re.search(r'^\s*&(.*)', line).group(1) == 'species':
                                countspec += 1
                                nml = re.search(r'^\s*&(.*)', line).group(1) + str(countspec)
                            else:
                                nml = re.search(r'^\s*&(.*)', line).group(1)
                            if nml not in self.namelists:
                                self.namelists.append(nml)
                    # Search lines for <parameter> = <value> patterns
                    p = re.compile(r'^\s*(.*)\s*=\s*(.*)')
                    m = p.search(line)
                    # Pick matching lines and build parameter dictionary
                    if m:
                        if m.group(1).strip() in self.spec_nl:
                            self.pardict[m.group(1).strip() + str(countspec)] = m.group(2)
                            self.nmldict[m.group(1).strip() + str(countspec)] = nml
                        else:
                            self.pardict[m.group(1).strip()] = m.group(2)
                            self.nmldict[m.group(1).strip()] = nml
        except IOError:
            print("Could not read parameters file")
            raise
        self._clean_parameters()
        self.add_defaults()

    def _clean_parameters(self):
        """ Clear the comments from all variables,

        Cast some strings to integers and floats
        """
        boolstr_t = [".T.", ".t.", "T", "t", ".true."]
        boolstr_f = [".F.", ".f.", "F", "f", ".false."]
        for item in self.pardict:
            self.pardict[item] = self.clearcomments(self.pardict[item])
            try:  # Can it be converted to int?
                self.pardict[item] = int(self.pardict[item])
            except ValueError:
                try:  # No, but can it be converted to float?
                    self.pardict[item] = float(self.pardict[item])
                except ValueError:
                    pass
            if self.pardict[item] in boolstr_t:  # cast switches to boolean values
                self.pardict[item] = True
            elif self.pardict[item] in boolstr_f:
                self.pardict[item] = False

    def add_defaults(self):
        """ Set default values GENE does not write

        Some diagnostics require these to exist at least in the named tuple
        """
        defpars = {"x0": 0.5, "ky0_ind": 0, "kx_center": 0.0,
                   "n0_global": 0,
                   "omega_prec": 1E-3, "coll": 0.0,
                   "Omega0_tor": 0.0, "ExBrate": 0.0,
                   "with_coriolis" : False, "with_centrifugal" : False,
                   "with_comoving_other" : False, "with_bxphi0" : False,
                   "sign_Ip_CW": 1, "sign_Bt_CW": 1, "n_pol": 1,
                   "Tref": 1.0, "nref": 1.0, "Bref": 1.0, "mref": 1.999,
                   "Lref": 1.0, "passive" : False,
                   "GIT_MASTER": '', "GIT_BRANCH": ''
               }
        defpnml = {"x0": "box", "ky0_ind": "box", "kx_center": "box",
                   "n0_global": "box",
                   "omega_prec": "general", "coll": "general",
                   "Omega0_tor" : "external_contr", "ExBrate" : "external_contr",
                   "with_coriolis" : "external_contr", "with_centrifugal" : "external_contr",
                   "with_comoving_other" : "external_contr", "with_bxphi0" : "external_contr",
                   "sign_Ip_CW": "geometry", "sign_Bt_CW": "geometry", "n_pol": "geometry",
                   "Tref": "units", "nref": "units", "Bref": "units", "mref": "units",
                   "Lref": "units", "passive": "species",
                   "GIT_MASTER": "info", "GIT_BRANCH": "info"
               }

        for defkey in defpars.keys():
            #special treatment for species namelist defaults
            if defpnml[defkey]=="species":
                for ispec in range(1,self.pardict['n_spec']+1):
                    self.pardict.setdefault(defkey+"{}".format(ispec), defpars[defkey])
                    self.nmldict.setdefault(defkey+"{}".format(ispec), defpnml[defkey]+"{}".format(ispec))
            else:
                self.pardict.setdefault(defkey, defpars[defkey])
                self.nmldict.setdefault(defkey, defpnml[defkey])
        try:
            minor_r = self.pardict["minor_r"]
        except KeyError:
            minor_r = 1
        rhostar = np.sqrt(
                self.pardict["Tref"]*self.pardict["mref"]*1.e3/1.60217733E-19*1.67262e-27) / \
                self.pardict["Bref"]/minor_r/self.pardict["Lref"]
        self.pardict.setdefault("rhostar", rhostar)

    def Write_Pars(self, path):
        """ Take the dict and write a GENE parameters file """
        parfileout = open(path, "w")
        specflag = False
        for item in self.namelists:
            specflag = False
            if item[0:-1] == 'species':
                specflag = True
                parfileout.write('&' + item[0:-1] + '\n')
            else:
                parfileout.write('&' + item + '\n')
            for par in self.pardict.keys():
                if self.nmldict[par] == item:
                    self._writeparameter(par, parfileout, specflag)
            parfileout.write('/\n\n')

    def _writeparameter(self, par, parfile, specflag):
        parvalue = str(self.pardict[par])
        # Take care of the different form of logic states in Python and Fortran
        if parvalue == "True":
            parvalue = ".T."
        elif parvalue == "False":
            parvalue = ".F."
        if specflag:
            parfile.write(par[0:-1] + ' = ' + parvalue + '\n')
        else:
            parfile.write(par + ' = ' + parvalue + '\n')

    def asnamedtuple(self):
        """ Return Parameters as a named tuple for easier usage """
        # We have to ignore parameters with whitespace (in info nml)
        valid = {}
        for item in self.pardict:
            if " " not in item:
                if '(' in item or ')' in item:
                    item_reduced = item.replace('(', '_')
                    item_reduced = item_reduced.replace(')', '_')
                    valid[item_reduced] = self.pardict[item]
                else:
                    valid[item] = self.pardict[item]
        return namedtuple('ParTuple', valid.keys())(*valid.values())



# -*- coding: utf-8 -*-
"""
Created on Thu Dec 12 12:49:43 2019

@author: dykua

Functions for handling credentials
"""

import pymongo
import pickle
import os

class mgk_login(object):
    def __init__(self, server='mongodb03.nersc.gov', port='27017', dbname='mgk_fusion', 
                 user='user', pwd = 'pwd'):

        self.login = {'server': server.strip(),
                      'port': port.strip(),
                      'dbname': dbname.strip(),
                      'user': user.strip(),
                      'pwd': pwd.strip()
                      }
        
    def from_saved(self, file_path):
        with open(file_path.strip(), 'rb') as pkl:
            info = pickle.load(pkl)
        
        self.login = info
        
        
    def save(self, file_path):
        
        with open(file_path, 'wb') as pkl:
            pickle.dump(self.login, pkl, protocol = pickle.HIGHEST_PROTOCOL)
        print('Login info saved to {}'.format(file_path) )    
            
    def update(self, dict_to_update):
        for key, val in dict_to_update.items():
            if key in self.login:
                self.login[key] = val 
    #   self.login.update(dict_to_update)
                
    def connect(self):
        ## Old way for pymongo 3.*
        # database = MongoClient(self.login['server'].strip(), int(self.login['port']) )[self.login['dbname'].strip()]
        # database.authenticate(self.login['user'].strip(), self.login['pwd'].strip())

        client = pymongo.MongoClient('mongodb://{user}:{pwd}@{server}:{port}/{dbname}'.format(**self.login),directConnection=True)
        database = client[self.login['dbname']]
        return client, database
        
def f_login_dbase(authenticate):

    info = authenticate

    if info is None:
        O1 = input("You did not enter credentials for accessing the database.\n You can \n 0: Enter it manually. \n 1: Enter the full path of the saved .pkl file\n")
        if O1 == '0':
            O2 = input("Please enter the server location, port, database name, username, password in order and separated by comma.\n").split(',')
            login = mgk_login(server= O2[0], port= O2[1], dbname=O2[2], user=O2[3], pwd = O2[4])
            O2_1 = input("You can save it by entering a target path, press ENTER if you choose not to save it\n")
            if len(O2_1)>1:
                login.save(os.path.abspath(O2_1) )
            else:
                print('Info not saved!')
                pass
        elif O1 == '1':
            O2= input("Please enter the target path\n")
            login = mgk_login()
            login.from_saved(os.path.abspath(O2))
        
        else:
            exit("Invalid input. Abort")
                
    else:
        login = mgk_login()
        try:
            login.from_saved(os.path.abspath(info))
        except OSError:
            exit("The specified credential file is not found!")

    return login
 


import numpy as np
from os.path import getsize

from .base_file import BinaryFile


class MomFile(BinaryFile):
    """ Class to parse binary mom_[spec].dat diagnostic output of GENE

    :param filename: mom file name
    :param common: CommonData object of the run to analyse
    """
    def __init__(self, filename, common, usemmap=False):
        super().__init__(filename, common, usemmap=usemmap)
        self.momvar3d = {"dens": None, "tpar": None, "tperp": None, "qpar": None, "qperp": None,
                         "upar": None, "n00": None, "n20": None, "n02": None}

    def _set_sizes(self):
        super()._set_sizes()
        self.nmoms = self.cm.pnt.n_moms
        self.entrysize = self.cm.pnt.nx0*self.cm.pnt.nky0*self.cm.pnt.nz0*self.complexsize
        # jumps in bytes in mom files
        self.leapmom = self.nmoms*(self.entrysize + 2*self.intsize)

    def reset_tinds(self):
        """ Reset time indices when reloading file"""
        super().reset_tinds()
        self.mtind = [-1]*self.nmoms

    def get_timearray(self):
        """Get time array for mom file """
        self.timearray = []
        self.file.seek(0)
        for i in range(int(getsize(self.filename)/(self.leapmom + self.tesize))):
            self.timearray.append(float(self.te.unpack(self.file.read(self.tesize))[1]))
            self.file.seek(self.leapmom, 1)

    def offset(self, var):
        """Calculate offset in mom file for a given self.time and variable"""
        if var in range(self.nmoms):
            leap = self.leapmom
            return self.tesize + self.tind*(self.tesize + leap) + var*(
                self.entrysize + 2*self.intsize) + self.intsize

    def readvar(self, var):
        """ Return 3d mom data at the time set in self.time"""
        var3d = self.readfunc(self.file, count=self.cm.pnt.nx0*self.cm.pnt.nky0*self.cm.pnt.nz0,
                            dtype=self.npct, offset=self.offset(var))
        if self.cm.x_local and not self.cm.y_local:  # y-global has yx order
            var3d = var3d.reshape(self.cm.pnt.nky0, self.cm.pnt.nx0, self.cm.pnt.nz0, order="F")
            var3d = np.swapaxes(var3d, 0, 1)
        else:
            var3d = var3d.reshape(self.cm.pnt.nx0, self.cm.pnt.nky0, self.cm.pnt.nz0, order="F")
        return var3d

    def get_var(self, varname):
        """ Return a 3d moment by name"""
        varidx = {"dens": 0, "tpar": 1, "tperp": 2, "qpar": 3, "qperp": 4, "upar": 5, "n00": 6, "n20": 7, "n02": 8}
        if not self.mtind[varidx[varname]] == self.tind:
                self.mtind[varidx[varname]] = self.tind
                self.momvar3d[varname] = self.readvar(varidx[varname])
        return self.momvar3d[varname]

    def dens(self):
        """ Return density fluctuation"""
        if not self.mtind[0] == self.tind:
            self.mtind[0] = self.tind
            self.dens3d = self.readvar(0)
        return self.dens3d

    def tpar(self):
        """ Return T_parallel fluctuations"""
        if not self.mtind[1] == self.tind:
            self.mtind[1] = self.tind
            self.tpar3d = self.readvar(1)
        return self.tpar3d

    def tperp(self):
        """ Return T_perp fluctuations"""
        if not self.mtind[2] == self.tind:
            self.mtind[2] = self.tind
            self.tperp3d = self.readvar(2)
        return self.tperp3d

    def qpar(self):
        """ Return q_parallel fluctuations"""
        if not self.mtind[3] == self.tind:
            self.mtind[3] = self.tind
            self.qpar3d = self.readvar(3)
        return self.qpar3d

    def qperp(self):
        """ Return q_perp fluctuations"""
        if not self.mtind[4] == self.tind:
            self.mtind[4] = self.tind
            self.qperp3d = self.readvar(4)
        return self.qperp3d

    def upar(self):
        """ Return parallel flow fluctuations"""
        if not self.mtind[5] == self.tind:
            self.mtind[5] = self.tind
            self.upar3d = self.readvar(5)
        return self.upar3d

    def n00(self):
        """ Return perpendicular pressure fluctuations"""
        if not self.mtind[6] == self.tind:
            self.mtind[6] = self.tind
            self.n00_3d = self.readvar(6)
        return self.n00_3d

    def n20(self):
        """ Return N20 fluctuations"""
        if not self.mtind[7] == self.tind:
            self.mtind[7] = self.tind
            self.n20_3d = self.readvar(7)
        return self.n20_3d

    def n02(self):
        """ Return N02 fluctuations"""
        if not self.mtind[8] == self.tind:
            self.mtind[8] = self.tind
            self.n02_3d = self.readvar(8)
        return self.n02_3d



"""PlotProfiles.py: Module to do the plotting of profile diagnostic outputput """

import numpy as np
import ..utils.averages as av
from ..utils import errors as err, nc_predictions
from .baseplot import plt, Plotting


def printaverageflux(prdlist, xmin, xmax):
    """ Print the (time averaged) mean heat flux for a radial domain

    :param prdlist: List of ProfileData objects to process
    :param xmin: Left boundary in x/L_ref
    :param xmax: Right boundary in x/L_ref
    """
    for prd in prdlist:
        for prs in prd.prspec:
            meanQnc = av.mytrapz(prs.dataarray["Qnc"], prs.timearray)
            errorQnc = err.windowerr(prs.dataarray["Qnc"], prs.timearray)[0]
            errorQt = err.windowerr(prs.dataarray["Qturb"], prs.timearray)[0]
            meanQt = av.mytrapz(prs.dataarray["Qturb"], prs.timearray)
            meanomn = av.mytrapz(prs.dataarray["omns"], prs.timearray)
            meanomt = av.mytrapz(prs.dataarray["omts"], prs.timearray)
            # Calculate the average fluxes at x=xmin-xmax and print them
            postoav = np.where(np.logical_and(prd.xs > xmin, prd.xs < xmax))
            meanQt_center = np.mean(meanQt[postoav])
            errorQt_center = np.mean(errorQt[postoav])
            meanomn_center = np.mean(meanomn[postoav])
            meanomt_center = np.mean(meanomt[postoav])
            print("Mean turbulent flux at {:2}-{:2}: {:5} +- {:5}".format(xmin, xmax, meanQt_center,
                                                                          errorQt_center))
            print("Mean density gradient at {:2}-{:2}: {:5}".format(xmin, xmax, meanomn_center))
            print("Mean temp gradient at {:2}-{:2}: {:5}".format(xmin, xmax, meanomt_center))
            meanQnc_center = np.mean(meanQnc[postoav])
            errorQnc_center = np.mean(errorQnc[postoav])
            print("Mean NC flux at {:2}-{:2}: {:5} +- {:5}".format(xmin, xmax, meanQnc_center,
                                                                   errorQnc_center))


def calc_fb_param(prd, fieldd):
    """Calculate the k in the radial force balance equation

    Uses time averaged quantities.
    :param prd: ProfileData object for n, T and the parameters file
    :param fieldd: FieldData object for E_r
    :returns: numpy array of k for each radial position
    """
    phi_ky0 = np.atleast_3d(np.real_if_close(fieldd.dataarray))
    erad_t = np.empty(phi_ky0.shape)
    for tind in range(len(fieldd.timearray)):
        erad_t[tind, :, 0] = -np.gradient(phi_ky0[tind, :, 0],
                                       (prd.xs[1] - prd.xs[0])/fieldd.cm.pnt.rhostar)
    erad = av.mytrapz(erad_t[..., 0], fieldd.timearray)
    fb_param = []
    epsilon = np.array(prd.xs*prd.cm.pnt.minor_r/prd.cm.pnt.major_R)
    btoverbp = prd.q/epsilon*np.sqrt(1 - epsilon ** 2)
    for prs in prd.prspec:
        times = prs.timearray
        jb_av = av.mytrapz(prs.dataarray["jbs"], times)
        ns_av = av.mytrapz(prs.dataarray["ns"], times)
        Ts_av = av.mytrapz(prs.dataarray["Ts"], times)
        omns_av = av.mytrapz(prs.dataarray["omns"], times)
        omts_av = av.mytrapz(prs.dataarray["omts"], times)
        fb_param.append(-(jb_av/(btoverbp*ns_av*Ts_av) - (omns_av + omts_av + erad/Ts_av))/omts_av)
    return fb_param


class PlotProfdata(Plotting):
    """ Plot time averaged profiles

    Generates plots of temperature and density, particle, heat and momentum flux,
    bootstrap current, radial force balance parameter, collisionality
    :param profdata: ProfileData object list to plot
    :param fielddata: FieldData object list to use in force balance plots
    :param plotset: Set which controls the plots
    """

    def __init__(self, profdata, fielddata, plotset):
        super().__init__()
        # Data structures used in the different plots
        self.prdlist = profdata
        self.fieldlist = fielddata
        self.plotset = plotset
        for prd in self.prdlist:
            if prd.isdatapresent is False:
                print("No profile data present in object ", prd.fext)
                self.plotset.clear()
        # Standard limits for the x axis are determined over all input runs
        self.xmin = min([prd.xs[1] for prd in self.prdlist])
        self.xmax = max([prd.xs[-1] for prd in self.prdlist])

    def output_avprof(self, avproffile_prefix):
        """ Generate time averaged density and temperature profiles
        which can be used as input for GENE

        :param avproffile_prefix: Output filename, species name is added to it
        """
        print("Writing averaged profile to file")
        for prd in self.prdlist:
            for n, prs in enumerate(prd.prspec):
                av_T = av.mytrapz(prs.Ts, prs.timearray)
                av_n = av.mytrapz(prs.ns, prs.timearray)
                avprofilename = (avproffile_prefix + "_" + prd.specname[n])
                with open(avprofilename, 'w') as avproffile:
                    avproffile.write(
                            "#   x/a             x/a(dummy)       T/Tref          n/nref\n")
                    avproffile.write("# Averaged profile generated by profile_data.py:"
                                     "t = {:4.1f} - {:4.1f}\n".format(prd.cm.pnt.starttime,
                                                                      prd.cm.pnt.endtime))
                    for i in range(0, prd.pnt.nx0):
                        avproffile.write(
                                '{:E}'.format(prd.xs[i]) + "\t" + '{:E}'.format(prd.xs[i]) + "\t")
                        avproffile.write(
                                '{:E}'.format(av_T[i]) + "\t" + '{:E}'.format(av_n[i]) + "\n")

    def createfigs(self, poutput):
        """ Generate the figures and axes objects to plot into for averaged profiles

        Calls the individual plotting routines
        Plots individual figures for each quantity and species, but puts their data from different
        runs into the same figures
        :param poutput: Switch to generate pdf file output
        """
        # Number of plotted variables
        numvars = 5
        # Number of plot windows
        nspecs = self.prdlist[0].cm.pnt.n_spec
        numfigs = (numvars - 1)*nspecs + 2
        figlist = [plt.figure() for _ in range(numfigs)]
        # One plot window for each species and quantity, one axis object in each
        axlist = [fig.add_subplot(111) for fig in figlist]
        # One legend per subplot
        leglist = [[] for _ in axlist]
        for iprd, prd in enumerate(self.prdlist):
            # Use the list of axes so that the different ProfileData objects end up in
            # the same figure and axis object (one per species)
            if 'plotT' in self.plotset:
                self.plot_tempdens(axlist[0:nspecs], prd)
            if 'plotQ' in self.plotset or 'plotG' in self.plotset or 'plotP' in self.plotset:
                self.plot_radflux(axlist[nspecs:2*nspecs], prd, "Qturb")
                self.plot_radflux(axlist[nspecs:2*nspecs], prd, "Qnc")
                # NC predictions only for first species (ions)
                self.plot_Qncpred(axlist[nspecs], prd)
                self.plot_radflux_SI(axlist[2*nspecs:3*nspecs], prd, "Qturb")
                self.plot_radflux_SI(axlist[2*nspecs:3*nspecs], prd, "Qnc")
            if 'plotj' in self.plotset:
                # Exception: Put all species in one axis
                self.plot_jbs([axlist[3*nspecs] for _ in range(nspecs)], prd, self.fieldlist[iprd])
                self.plot_fb_param([axlist[3*nspecs + 1] for _ in range(nspecs)], prd,
                                   self.fieldlist[iprd])
            axcount = 3*nspecs + 2
            if 'plotnu' in self.plotset:
                self.plot_nustar(axlist[axcount:axcount + nspecs], prd)
            self.shade_buffer_region(axlist, prd)
        for ax in axlist:
            ax.legend()
        if poutput:
            figlist[0].savefig("profiles{}.pdf".format(self.prdlist[0].fext))
            figlist[1].savefig("radfluxes{}.pdf".format(self.prdlist[0].fext))
            figlist[2].savefig("radareaflux{}.pdf".format(self.prdlist[0].fext))
            figlist[3].savefig("j_botstrap{}.pdf".format(self.prdlist[0].fext))
            figlist[4].savefig("nustar{}.pdf".format(self.prdlist[0].fext))

    @staticmethod
    def shade_buffer_region(axlist, prd):
        ubuff = 0
        lbuff = 0
        try:
            if prd.pnt.lcoef_krook:
                lbuff = prd.pnt.l_buffer_size
        except AttributeError:  # No buffer zone set
            pass
        try:
            if prd.pnt.ucoef_krook:
                ubuff = prd.pnt.u_buffer_size
        except AttributeError:
            pass
        for ax in axlist:
            if lbuff > 0:
                ax.axvspan(prd.xs[0], prd.xs[0] + lbuff, alpha=0.4, color="Black")
            if ubuff > 0:
                ax.axvspan(prd.xs[-1] - ubuff, prd.xs[-1], alpha=0.4, color="Black")

    def plot_tempdens(self, axlist, prd):
        """Plot the temperature and density profiles

        Profiles are temporally averaged by the trapezoid rule
        if the lists contain more than one time block.
        The initial profiles are given as reference.

        :param axlist: List of axes objects to plot into (one per species)
        :param prd: ProfileData object containing the data
        """
        for iprs, prs in enumerate(prd.prspec):
            baseomn, = axlist[iprs].plot(prd.xs, av.mytrapz(prs.dataarray["omns"], prs.timearray),
                                         label=r'$a/L_n$')
            baseomt, = axlist[iprs].plot(prd.xs, av.mytrapz(prs.dataarray["omts"], prs.timearray),
                                         label=r'$a/L_T$')
            basen, = axlist[iprs].plot(prd.xs, av.mytrapz(prs.dataarray["ns"], prs.timearray),
                                       label=r'$n$')
            baset, = axlist[iprs].plot(prd.xs, av.mytrapz(prs.dataarray["Ts"], prs.timearray),
                                       label=r'$T$')
            axlist[iprs].plot(prd.xs, prd.omn0s[:, 0], '--', color=baseomn.get_color())
            axlist[iprs].plot(prd.xs, prd.omt0s[:, 0], '--', color=baseomt.get_color())
            axlist[iprs].plot(prd.xs, prd.T0s[:, 0]*prd.cm.pars["temp" + str(iprs + 1)], '--',
                              color=baset.get_color())
            axlist[iprs].plot(prd.xs, prd.n0s[:, 0]*prd.cm.pars["dens" + str(iprs + 1)], '--',
                              color=basen.get_color())
            axlist[iprs].set_xlabel(r'$x/a$')
            axlist[iprs].set_xlim(self.xmin, self.xmax)
            axlist[iprs].set_title(prd.cm.specnames[iprs])

    def plot_radflux(self, axlist, prd, quantity, linestyle="-"):
        """Plot the averaged fluxes

         Plots the time averaged particle, heat and momentum fluxes.

         :param axlist: List of axes objects (one per each species)
         :param prd: ProfileData object containing the data
         :param quantity: Name of the flux to plot
         :param linestyle: See matplotlib documentation for possible forms
         """
        linelabel = {"Gammaturb": r"$\Gamma_{{\mathrm{{turb}}}}$",
                     "Gammanc": r"$\Gamma_{{\mathrm{{NC}}}}$", "Qturb": r"$Q_{{\mathrm{{turb}}}}$",
                     "Qnc": r"$Q_{{\mathrm{{NC}}}}$", "Piturb": r"$\Pi_{{\mathrm{{turb}}}}$",
                     "Pinc": r"$\Pi_{{\mathrm{{NC}}}}$"}
        for n, prs in enumerate(prd.prspec):
            mean_quant = av.mytrapz(prs.dataarray[quantity], prs.timearray)
            error_quant = err.windowerr(prs.dataarray[quantity], prs.timearray)[0]
            base_line, = axlist[n].plot(prd.xs, mean_quant, linestyle=linestyle,
                                        label=linelabel[quantity] + "{}".format(
                                                prd.cm.specnames[n]))
            axlist[n].fill_between(prd.xs, mean_quant - error_quant, mean_quant + error_quant,
                                   alpha=0.2, facecolor=base_line.get_color())
            axlist[n].set_xlim(self.xmin, self.xmax)
            axlist[n].set_xlabel(r'$x/a$')
            self._set_flux_y_label(axlist[n], quantity)

    def _set_flux_y_label(self, axesobj, quantity, SI=False):
        """ Set the y axis label to the right type of gyro-Bohm or SI unit"""
        if SI:
            gbaxlabel = {"Gammaturb": r"$10^19/\mathrm{m^2s}$", "Gammanc": r"$10^19/\mathrm{m^2s}$",
                         "Qturb": r"$\mathrm{kW/m^2}$", "Qnc": r"$\mathrm{kW/m^2}$",
                         "Piturb": r"$\mathrm{kg/m s^2}$", "Pinc": r"$\mathrm{kg/m s^2}$"}
        else:
            gbaxlabel = {"Gammaturb": r"$\Gamma/\Gamma_{gB}$", "Gammanc": r"$\Gamma/\Gamma_{gB}$",
                         "Qturb": r"$Q/Q_{gB}$", "Qnc": r"$Q/Q_{gB}$", "Piturb": r"$\Pi/\Pi_{gB}$",
                         "Pinc": r"$\Pi/\Pi_{gB}$"}
        currlabel = axesobj.get_ylabel()
        if gbaxlabel[quantity] not in currlabel:
            currlabel += gbaxlabel[quantity]
        axesobj.set_ylabel(currlabel)

    @staticmethod
    def plot_Qncpred(axis, prd, potato=False):
        """ Plot neoclassical predictions based on several (semi-)analytic formulas

        Currently provides Chang-Hinton and potato region models from Lin and Bergmann.
        :param axis: A single axes object (should be the main ions, Chang-Hinton might not apply
        for anything else)
        :param prd:  ProfileData object containing the data
        :param potato: Include potato regime predictions
        """
        meanch = av.mytrapz(prd.Qcharr, prd.prspec[0].timearray)
        axis.plot(prd.xs, meanch, '--', label=r"Chang-Hinton")
        if potato:
            potwidth = nc_predictions.calc_potwidth(prd)
            x0 = np.array(0.25*(prd.xs/potwidth*(1 + np.sqrt(5))) ** 3)
            axis.plot(prd.xs, meanch*(1 - np.exp(-x0)*(x0 + 1)), '-.', label=r'Lin')
            rpind3 = np.where(prd.xs <= 3*potwidth)
            axis.plot(prd.xs[rpind3], meanch[rpind3]*(1 - (1 - x0[rpind3] ** (1.0/3)/3) ** 2),
                      label=r'Bergmann')

    def plot_radflux_SI(self, axlist, prd, quantity, linestyle="-"):
        """Plot the averaged fluxes in SI units

         Plots the time averaged particle, heat and momentum fluxes.

         :param axlist: List of axes objects (one per each species)
         :param prd: ProfileData object containing the data
         :param quantity: Name of the flux to plot
         :param linestyle: See matplotlib documentation for possible forms
         """
        linelabel = {"Gammaturb": r"$\Gamma_{{\mathrm{{turb}}}}$",
                     "Gammanc": r"$\Gamma_{{\mathrm{{NC}}}}$", "Qturb": r"$Q_{{\mathrm{{turb}}}}$",
                     "Qnc": r"$Q_{{\mathrm{{NC}}}}$", "Piturb": r"$\Pi_{{\mathrm{{turb}}}}$",
                     "Pinc": r"$\Pi_{{\mathrm{{NC}}}}$"}
        for n, prs in enumerate(prd.prspec):
            mean_quant = av.mytrapz(prs.dataarray[quantity], prs.timearray)
            mean_quant_SI = mean_quant*self.gyrobohm_SI(prd.cm, quantity)
            error_quant = err.windowerr(prs.dataarray[quantity], prs.timearray)[0]
            error_quant_SI = error_quant*self.gyrobohm_SI(prd.cm, quantity)
            base_line, = axlist[n].plot(prd.xs, mean_quant_SI, linestyle=linestyle,
                                        label=linelabel[quantity] + "{}".format(
                                                prd.cm.specnames[n]))
            axlist[n].fill_between(prd.xs, mean_quant_SI - error_quant_SI,
                                   mean_quant_SI + error_quant_SI, alpha=0.2,
                                   facecolor=base_line.get_color())
            axlist[n].set_xlim(self.xmin, self.xmax)
            axlist[n].set_xlabel(r'$x/a$')
            self._set_flux_y_label(axlist[n], quantity, SI=True)

    @staticmethod
    def renorm_profile(prd, ispec, quantity):
        """ Renormalization factor for a radial profile to local gyro-Bohm units

        i.e. gB units of each radial position instead of the gB at x0 position
        :param prd: ProfileData object to get temperature and density from
        :param ispec: Species index for prof
        :param quantity: Type of prof (e.g. Gammaturb, Qnc)
        """
        renorm = {"Ts": prd.T0s[:, ispec], "ns": prd.n0s[:, ispec], "omts": 1,
                  # The T,n  dependency in omt, omn compensates itself
                  "omns": 1, "Gammanc": prd.T0s[:, ispec] ** (3/2)*prd.n0s[:, ispec],
                  "Gammaturb": prd.T0s[:, ispec] ** (3/2)*prd.n0s[:, ispec],
                  "Qturb": prd.T0s[:, ispec] ** (5/2)*prd.n0s[:, ispec],
                  "Qnc": prd.T0s[:, ispec] ** (5/2)*prd.n0s[:, ispec],
                  "Pinc": prd.T0s[:, ispec] ** 2*prd.n0s[:, ispec],
                  "Piturb": prd.T0s[:, ispec] ** 2*prd.n0s[:, ispec],
                  "jbs": prd.T0s[:, ispec]*prd.n0s[:, ispec]}
        return renorm[quantity]

    def plot_jbs(self, axlist, prd, fieldd):
        """Plot the species-wise bootstrap current

        Plots time averaged < u_par B >_FS for every species.
        The bootstrap current would be their sum
        :param axlist: List of axes objects (one per each species)
        :param prd: ProfileData object containing the data
        :param fieldd: FieldData object for radial force balance
        """
        for n, prs in enumerate(prd.prspec):
            axlist[n].plot(prd.xs, av.mytrapz(prs.dataarray["jbs"], prs.timearray),
                           label=r"$j_{BS}$")
            phi_ky0 = np.real_if_close(fieldd.dataarray)
            erad_t = np.empty(phi_ky0.shape)
            #for tind in range(len(fieldd.timearray)):
            #    erad_t[tind, :] = -np.gradient(phi_ky0[tind, :],
            #                                   (prd.xs[1] - prd.xs[0])/prd.cm.pnt.rhostar)
            axlist[n].plot(prd.xs, av.mytrapz(erad_t, fieldd.timearray), label=r"$E_r$")
            axlist[n].set_xlabel(r'$x/a$')
            axlist[n].set_xlim(self.xmin, self.xmax)
            axlist[n].set_ylabel(r'$j_{BS}$')

    def plot_fb_param(self, axlist, prd, fieldd):
        """Plot the force balance parameter k

        Plots time averaged < u_par B >_FS for every species.
        The bootstrap current would be their sum
        :param axlist: List of axes objects (one per each species)
        :param prd: ProfileData object containing the data
        :param fieldd: FieldData object corresponding to prd
        """
        fb_param = calc_fb_param(prd, fieldd)
        for n, prs in enumerate(prd.prspec):
            axlist[n].plot(prd.xs, fb_param[n], label="GENE")
            axlist[n].plot(prd.xs, nc_predictions.fb_param_hinton_haz(prd), ":", label="H-H")
            axlist[n].plot(prd.xs, nc_predictions.fb_param_hirsch_sig(prd), "--", label="H-S")
            axlist[n].set_xlabel(r'$x/a$')
            axlist[n].set_ylabel(r'$k$')
            axlist[n].set_xlim(self.xmin, self.xmax)
            axlist[n].set_xlim(0.25, 0.85)
            axlist[n].set_ylim(0, 1.3)

    def plot_nustar(self, axlist, prd):
        """ Plot the collisionality and the inverse aspect ratio for comparison

        :param axlist: List of axes objects (one per each species)
        :param prd: ProfileData object containing the data
        """
        eps = prd.xs*prd.cm.pnt.minor_r/prd.cm.pnt.major_R
        for n, prs in enumerate(prd.prspec):
            axlist[n].plot(prd.xs, av.mytrapz(prs.nustar, prs.timearray)*eps ** 1.5,
                           label=r"$\nu_* \cdot \epsilon^{3/2}$ ")
            axlist[n].plot(prd.xs, eps ** 1.5, label=r"$\epsilon^{3/2}$")
            # B \circ Bref near the mag axis
            potwidth = nc_predictions.calc_potwidth(prd, n + 1)
            trapfr = (potwidth/prd.cm.pnt.major_R) ** 0.5
            print("Trapping fraction (potato):", trapfr)
            print("Potato width:", potwidth)
            axlist[n].hlines(trapfr ** 3, 0, potwidth)
            mspec = prd.cm.pars["mass{}".format(n + 1)]
            colltmp = []
            for tb, nu in enumerate(prs.nustar):
                colltmp.append((nu/prd.q/prd.cm.pnt.major_R*np.sqrt(
                        2*prd.prspec[0].dataarray["Ts"][tb]/mspec)*eps ** 1.5) ** (-1))
            # This is tau_ii as defined in Helander/Sigmar, note tau_i = sqrt(2) * tau_ii
            colltime = av.mytrapz(colltmp, prs.timearray)
            axlist[n].text(0.1, 0.5, "Highest collision time: {}".format(np.max(colltime)))
            axlist[n].text(0.1, 0.3, "Lowest collision time: {}".format(np.min(colltime)))
            axlist[n].set_yscale('log')
            axlist[n].set_xlabel(r'$x/a$')
            axlist[n].set_xlim(self.xmin, self.xmax)

    @classmethod
    def show(cls):
        plt.show()


class PlotProfxt(Plotting):
    """Plot contours of profiles and fluxes in x-t space.

    :param profdata: List of ProfileData objects to plot
    """

    def __init__(self, profdata):
        super().__init__()
        self.prdlist = profdata
        for prd in self.prdlist:
            if prd.isdatapresent is False:
                print("No profile data present in object ", prd.cm.fext)
        self.xmin = min([prd.xs[1] for prd in self.prdlist])
        self.xmax = max([prd.xs[-1] for prd in self.prdlist])

    def createfigs(self, poutput):
        """ Generate the figures and axes objects to plot into.

        Calls the individual plotting routines
        Currently creates one window per quantity and per species with subplots for each run
        given in the profdata list
        :param poutput: Switch to generate pdf output files
        """
        # Number of plotted variables
        numvars = 3
        # Number of plot windows
        numfigs = numvars*len(self.prdlist[0].prspec)
        # Number of subplots per window
        numsub = len(self.prdlist)
        figlist = [plt.figure() for _ in range(numfigs)]
        # One plot window for each species, but subplots for the different input runs
        axlist = [[] for _ in range(numfigs)]
        # Define a subplot grid which leaves room for the colorbar
        gs = plt.GridSpec(2, numsub, height_ratios=[12, 1])
        for ifig, fig in enumerate(figlist):
            axlist[ifig] = [fig.add_subplot(gs[0, isub]) for isub in range(numsub)]
        for iprd, prd in enumerate(self.prdlist):
            prdax = [axlist[i][iprd] for i in range(numfigs)]
            omtcm = self.plotprof_cont(prdax[0:numfigs//numvars], prd, quantity="omts")
            fluxcm = self.plotprof_cont(prdax[numfigs//numvars:2*numfigs//numvars], prd, "Qturb")
            jbscm = self.plotprof_cont(prdax[2*numfigs//numvars: 3*numfigs//numvars], prd, "jbs")
        # Generate colorbars, currently using the first set
        figlist[0].colorbar(omtcm[0], cax=figlist[0].add_subplot(gs[1, :]),
                            orientation="horizontal")
        figlist[1].colorbar(fluxcm[0], cax=figlist[1].add_subplot(gs[1, :]),
                            orientation="horizontal")
        figlist[2].colorbar(jbscm[0], cax=figlist[2].add_subplot(gs[1, :]),
                            orientation="horizontal")
        if poutput:
            figlist[0].savefig('profiles_xt{}.pdf'.format(self.prdlist[0].fext))
            figlist[1].savefig('radfluxes_xt{}.pdf'.format(self.prdlist[0].fext))
            figlist[2].savefig('jbs_xt{}.pdf'.format(self.prdlist[0].fext))

    def plotprof_cont(self, axlist, prd, quantity, vmin=0, vmax=0):
        """ Plot contour of the temperature, density or their gradient.

        :param axlist: List of axes that should have the length of the number of species
         of the second arg
        :param prd: ProfileData object which is to be plotted
        :param vmin: Minimum for the colormap
        :param vmax: Maximum for the colormap
        :param quantity: which of the fields in ProfileData to plot
        :returns: List of plotted colormaps (for colorbar)
        """
        cmlist = []
        for iprs, prs in enumerate(prd.prspec):
            axlist[iprs].set_xlim([np.min(prd.xs), np.max(prd.xs)])
            axlist[iprs].set_ylim([np.min(prs.timearray), np.max(prs.timearray)])
            colormap = self._set_colormap(quantity)
            if not (vmin == 0 and vmax == 0):
                cmlist.append(axlist[iprs].pcolormesh(prd.xs, prs.timearray,
                                                      np.array(prs.dataarray[quantity]),
                                                      shading="gouraud", cmap=colormap,
                                                      rasterized=True, vmin=vmin, vmax=vmax))
            else:
                cmlist.append(axlist[iprs].pcolormesh(prd.xs, prs.timearray,
                                                      np.array(prs.dataarray[quantity]),
                                                      shading="gouraud", cmap=colormap,
                                                      rasterized=True))
            axlist[iprs].set_title(self.titles[quantity], y=1.03)
            axlist[iprs].set_xlabel(r"$x/a$")
            axlist[iprs].set_ylabel(r"$t/(a/c_s)$")
        return cmlist

    def _set_colormap(self, quantity):
        cmapdict = {"Ts": self.cmap_unidirect, "ns": self.cmap_unidirect,
                    "omts": self.cmap_unidirect, "omns": self.cmap_unidirect,
                    "Gammanc": self.cmap_bidirect, "Gammaturb": self.cmap_bidirect,
                    "Qturb": self.cmap_unidirect, "Qnc": self.cmap_unidirect,
                    "Pinc": self.cmap_bidirect, "Piturb": self.cmap_bidirect,
                    "jbs": self.cmap_bidirect}
        return cmapdict[quantity]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import h5py
import matplotlib.pyplot as plt
import numpy as np

import ..utils.averages as averages
from .baseplot import Plotting
from .diagnostic import Diagnostic


class DiagProfiles(Diagnostic):
    def __init__(self, avail_vars=None, specnames=None):
        super().__init__()
        self.name = 'Profiles'
        self.tabs = ['xglobal', 'GENE3D']

        self.help_txt = """Plots profiles for each species
                        \nSave h5 : save hdf5 file (def. True)"""

        self.avail_vars = avail_vars
        self.specnames = specnames

        self.opts = {"spec": {'tag': "Spec", 'values': self.list_specnames()},
                     "t_avg": {'tag': "t avg", 'values': [True, False]},
                     "save_h5": {"tag": "Save h5", "values": [True, False]}}

        self.set_defaults()

        self.options_gui = Diagnostic.OptionsGUI()

    def set_options(self, run_data, species):

        class ProfilesStep:
            def __init__(self, specnames):
                for spec in specnames:
                    setattr(self, spec, self.__Specprofiles())

            class __Specprofiles:
                def __init__(self):
                    self.T = []
                    self.n = []
                    self.u = []

        self.run_data = run_data

        self.geom = run_data.geometry

        self.rhostarref = self.run_data.pars["rhostar"] * self.run_data.pars["minor_r"]

        self.profiles_step = ProfilesStep(self.specnames)

        self.get_needed_vars(['mom'])

        self.get_spec_from_opts()

        return self.needed_vars

    def execute(self, data, step):
        for spec in self.specnames:

            profiles_step = getattr(self.profiles_step, spec)

            if self.run_data.flux_tube:
                #   flux tube
                dens = getattr(getattr(data, 'mom_' + spec), "dens")(step.time, step.mom)
                T_par = getattr(getattr(data, 'mom_' + spec), "T_par")(step.time, step.mom)
                T_perp = getattr(getattr(data, 'mom_' + spec), "T_perp")(step.time, step.mom)
                u_par = getattr(getattr(data, 'mom_' + spec), "u_par")(step.time, step.mom)

                profiles_step.T.append(np.average(np.squeeze(1/3*T_par[:, 0, :]+2/3*T_perp[:, 0, :]),
                                              weights=self.J_norm, axis=-1))

                profiles_step.n.append(np.average(np.squeeze(dens[:, 0, :]),
                                                  weights=self.J_norm, axis=-1))

                profiles_step.u.append(np.average(np.squeeze(u_par[:, 0, :]),
                                                  weights=self.J_norm, axis=-1))
            elif self.run_data.x_global:
                # x global
                dens = getattr(getattr(data, 'mom_' + spec), "dens")(step.time, step.mom)
                T_par = getattr(getattr(data, 'mom_' + spec), "T_par")(step.time, step.mom)
                T_perp = getattr(getattr(data, 'mom_' + spec), "T_perp")(step.time, step.mom)
                u_par = getattr(getattr(data, 'mom_' + spec), "u_par")(step.time, step.mom)

                profiles_step.T.append(np.average(np.squeeze(1/3 * T_par[:, 0, :] + 2/3 * T_perp[:, 0, :]),
                                                  weights=self.geom.jacobian.T, axis=-1))

                profiles_step.n.append(np.average(np.squeeze(dens[:, 0, :]),
                                                  weights=self.geom.jacobian.T, axis=-1))

                profiles_step.u.append(np.average(np.squeeze(u_par[:, 0, :]),
                                                  weights=self.geom.jacobian.T, axis=-1))
            elif self.run_data.is3d:
                # 3D
                dens = getattr(getattr(data, 'mom_' + spec), "n")(step.time, step.mom)
                T_par = getattr(getattr(data, 'mom_' + spec), "T_par")(step.time, step.mom)
                T_perp = getattr(getattr(data, 'mom_' + spec), "T_per")(step.time, step.mom)
                u_par = getattr(getattr(data, 'mom_' + spec), "u_par")(step.time, step.mom)

                profiles_step.T.append(np.average(np.squeeze(1/3 * T_par + 2/3 * T_perp),
                                                  weights=self.geom.jacobian.T, axis=(1, 2)))

                profiles_step.n.append(np.average(np.squeeze(dens),
                                                  weights=self.geom.jacobian.T, axis=(1, 2)))

                profiles_step.u.append(np.average(np.squeeze(u_par),
                                                  weights=self.geom.jacobian.T, axis=(1, 2)))
            else:
                raise NotImplementedError("Profiles calculation ")

    def plot(self, time_requested, output=None, out_folder=None):
        def plot_a_map(ax, x, y, f, x_lbl, y_lbl, ttl):
            cm1 = ax.pcolormesh(x, y, f)
            ax.set_rasterization_zorder(z=-10)
            ax.set_xlabel(x_lbl)
            ax.set_ylabel(y_lbl)
            ax.set_title(ttl)
            fig.colorbar(cm1)

        self.plotbase = Plotting()

        if self.run_data.x_global or self.run_data.is3d:
            y_lbl = r'$x/a$'
            y_ax = self.run_data.spatialgrid.x_a
        else:
            y_lbl = r'$x/\rho_{ref}$'
            y_ax = self.run_data.spatialgrid.x

        x_lbl = r'$t\;c_{ref}/L_{ref}$'

        for i_s, spec in enumerate(self.specnames):
            omn_b = self.run_data.profilesdata.omn0s[:, i_s]
            omt_b = self.run_data.profilesdata.omt0s[:, i_s]
            T_b = self.run_data.profilesdata.T0s[:, i_s]/self.run_data.pars["temp" + spec]/self.run_data.pars["Tref"]
            n_b = self.run_data.profilesdata.n0s[:, i_s]/self.run_data.pars["dens" + spec]/self.run_data.pars["nref"]

            temp = np.array(getattr(getattr(self.profiles_step, spec), 'T')) * self.rhostarref
            n = np.array(getattr(getattr(self.profiles_step, spec), 'n')) * self.rhostarref
            u = np.array(getattr(getattr(self.profiles_step, spec), 'u')) * self.rhostarref

            omt = -np.gradient(np.log(T_b + temp), self.run_data.spatialgrid.x_a, axis=1) / \
                self.run_data.pars["minor_r"]
            omn = -np.gradient(np.log(n_b + n), self.run_data.spatialgrid.x_a, axis=1) / \
                self.run_data.pars["minor_r"]


            if len(time_requested) > 1:
                fig = plt.figure(figsize=(15, 10))
                ax = fig.add_subplot(2, 3, 1)
                plot_a_map(ax, time_requested, y_ax,
                           n.T, x_lbl, y_lbl, r"$\delta n " + spec + "$")
                ax = fig.add_subplot(2, 3, 4)
                plot_a_map(ax, time_requested, y_ax,
                           omn.T, x_lbl, y_lbl, r"$omn " + spec + "$")

                ax = fig.add_subplot(2, 3, 2)
                plot_a_map(ax, time_requested, y_ax,
                           temp.T, x_lbl, y_lbl, r"$\delta T " + spec + "$")
                ax = fig.add_subplot(2, 3, 5)
                plot_a_map(ax, time_requested, y_ax,
                           omt.T, x_lbl, y_lbl, r"$ omt " + spec + "$")
                ax = fig.add_subplot(2, 3, 3)
                plot_a_map(ax, time_requested, y_ax,
                           u.T, x_lbl, y_lbl, r"$\delta u_{//} " + spec + "$")

                fig.tight_layout()
                fig.show()

            if self.opts['t_avg']['value'] and len(time_requested) > 1:
                # time averaged values
                fig = plt.figure(figsize=(15, 10))
                ax = fig.add_subplot(2, 3, 1)
                ax.plot(y_ax, n_b + averages.mytrapz(n, time_requested))
                ax.plot(y_ax, n_b, ls='--')
                ax.set_xlabel(y_lbl)
                ax.set_title(r"$n {}$".format(spec))

                ax = fig.add_subplot(2, 3, 2)
                ax.plot(y_ax, T_b + averages.mytrapz(temp, time_requested))
                ax.plot(y_ax, T_b, ls='--')
                ax.set_xlabel(y_lbl)
                ax.set_title(r"$T {}$".format(spec))

                ax = fig.add_subplot(2, 3, 3)
                ax.plot(y_ax, averages.mytrapz(u, time_requested))
                ax.set_xlabel(y_lbl)
                ax.set_title(r"$\delta u {}$".format(spec))

                ax = fig.add_subplot(2, 3, 4)
                ax.plot(y_ax, averages.mytrapz(omn, time_requested))
                ax.plot(y_ax, omn_b, ls="--")
                ax.set_xlabel(y_lbl)
                ax.set_title(r"$omn {}$".format(spec))

                ax = fig.add_subplot(2, 3, 5)
                ax.plot(y_ax, averages.mytrapz(omt, time_requested))
                ax.plot(y_ax, omt_b, ls="--")
                ax.set_xlabel(y_lbl)
                ax.set_title(r"$omT {}$".format(spec))

                fig.tight_layout()
                fig.show()

                if self.opts['save_h5']['value']:
                    file = h5py.File(out_folder / 'profile_{}.h5'.format(spec), 'w')
                    file["/x_a"] = y_ax
                    file["/n_" + spec] = n_b + averages.mytrapz(n, time_requested)
                    file["/T_" + spec] = n_b + averages.mytrapz(temp, time_requested)
                    file["/omn_" + spec] = averages.mytrapz(omn, time_requested)
                    file["/omT_" + spec] = averages.mytrapz(omt, time_requested)
                    file.close()
